@@ -144,6 +144,7 @@ TAB(MoveToColumn),
 TAB(BlockPasteStream),
 TAB(BlockPasteLine),
 TAB(BlockPasteColumn),
+TAB(BlockPasteOver),
 TAB(ShowPosition),
 
 TAB(FoldCreate),
@@ -251,6 +252,8 @@ TAB(ToggleBackSpUnindents),
 TAB(ToggleWordWrap),
 TAB(ToggleTrim),
 TAB(ToggleShowMarkers),
+TAB(ToggleHilitTags),
+TAB(ToggleShowBookmarks),
 TAB(SetLeftMargin),
 TAB(SetRightMargin),
 TAB(SetPrintDevice),
@@ -267,6 +270,10 @@ TAB(DirGoUp),
 TAB(DirGoDown), 
 TAB(DirGoRoot),
 TAB(DirGoto),
+TAB(DirSearchCancel),
+TAB(DirSearchNext),
+TAB(DirSearchPrev),
+TAB(DeleteFile),
 TAB(ShowVersion),
 TAB(ASCIITable),
 TAB(TypeChar),
@@ -278,6 +285,9 @@ TAB(DesktopSaveAs),
 TAB(ChildClose),
 TAB(BufListFileSave),
 TAB(BufListFileClose),
+TAB(BufListSearchCancel),
+TAB(BufListSearchNext),
+TAB(BufListSearchPrev),
 TAB(ViewModeMap),
 TAB(ClearMessages),
 TAB(BlockUnTab),
@@ -324,12 +334,29 @@ TAB(SetIndentWithTabs),
 TAB(RunCompiler),
 TAB(FoldCreateAtRoutines),
 TAB(LineCenter),
-TAB(RunProgramAsync)
-};
+TAB(RunProgramAsync),
+
+TAB(ListMark),
+TAB(ListUnmark),
+TAB(ListToggleMark),
+TAB(ListMarkAll),
+TAB(ListUnmarkAll),
+TAB(ListToggleMarkAll),
+
+TAB(Cvs),
+TAB(RunCvs),
+TAB(ViewCvs),
+TAB(ClearCvsMessages),
+TAB(CvsDiff),
+TAB(RunCvsDiff),
+TAB(ViewCvsDiff),
+TAB(CvsCommit),
+TAB(RunCvsCommit),
+TAB(ViewCvsLog)
 #if 0
 //TAB(ShowMsg),
 TAB(BlockReadPipe),
 TAB(BlockWritePipe),
 TAB(BlockPipe),
-};
 #endif
+};
