@@ -10,6 +10,8 @@
 #ifndef __HILIT_H__
 #define __HILIT_H__
 
+#include "e_regex.h"
+
 typedef unsigned short hlState;
 typedef unsigned char hsState;
 
@@ -32,9 +34,6 @@ HILIT_P(Hilit_Plain);
 #ifdef CONFIG_HILIT_C
 HILIT_P(Hilit_C);
 #endif
-#ifdef CONFIG_HILIT_HTML
-HILIT_P(Hilit_HTML);
-#endif
 #ifdef CONFIG_HILIT_PERL
 HILIT_P(Hilit_PERL);
 #endif
@@ -44,12 +43,6 @@ HILIT_P(Hilit_MAKE);
 #ifdef CONFIG_HILIT_REXX
 HILIT_P(Hilit_REXX);
 #endif
-#ifdef CONFIG_HILIT_DIFF
-HILIT_P(Hilit_DIFF);
-#endif
-#ifdef CONFIG_HILIT_MERGE
-HILIT_P(Hilit_MERGE);
-#endif
 #ifdef CONFIG_HILIT_IPF
 HILIT_P(Hilit_IPF);
 #endif
@@ -99,6 +92,9 @@ int Indent_SIMPLE(EBuffer *B, int Line, int PosCursor);
 #    define PCLI unsigned char
 #endif
 
+#define HILIT_CLRD() \
+    ((Color < COUNT_CLR) ? Colors[Color] : Color - COUNT_CLR)
+
 #define ColorChar() \
     do {\
     BPos = C - Pos; \
@@ -106,7 +102,7 @@ int Indent_SIMPLE(EBuffer *B, int Line, int PosCursor);
     if (BPos >= 0 && BPos < Width) { \
     BPtr = (PCLI *) (B + BPos); \
     BPtr[0] = *p; \
-    BPtr[1] = Color; \
+    BPtr[1] = HILIT_CLRD(); \
     } \
     if (StateMap) StateMap[i] = (hsState)(State & 0xFF); \
     } while (0)
@@ -117,10 +113,49 @@ int Indent_SIMPLE(EBuffer *B, int Line, int PosCursor);
 #define NextChar() do { i++; p++; len--; C++; } while (0)
 #define ColorNext() do { ColorChar(); NextChar(); } while (0)
 
-#define HILIT_VARS(ColorNormal, Line) \
+#define UntilMatchBrace(first, cmd) \
+    do { \
+        int Count[] = { 0, 0, 0, }; \
+        switch (first) \
+        { \
+            case '{': ++Count[0]; break; \
+            case '[': ++Count[1]; break; \
+            case '(': ++Count[2]; break; \
+        } \
+\
+        while (len > 0)        \
+        {                      \
+            switch (*p) {      \
+            case '{':          \
+                ++Count[0];    \
+                break;         \
+            case '}':          \
+                --Count[0];    \
+                break;         \
+            case '[':          \
+                ++Count[1];    \
+                break;         \
+            case ']':          \
+                --Count[1];    \
+                break;         \
+            case '(':          \
+                ++Count[2];    \
+                break;         \
+            case ')':          \
+                --Count[2];    \
+                break;         \
+            }                  \
+            cmd;               \
+            if (TEST_ZERO)     \
+                break;         \
+        } \
+    } while (0)
+
+#define HILIT_VARS(ColorTable, Line) \
     PCLI *BPtr; \
     int BPos; \
-    ChColor Color = ColorNormal; \
+    ChColor *Colors = ColorTable; \
+    ChColor Color = CLR_Normal; \
     int i; \
     int len = Line->Count; \
     char *p = Line->Chars; \
@@ -136,14 +171,24 @@ int Indent_SIMPLE(EBuffer *B, int Line, int PosCursor);
     if (*p == '\t' && ExpandTabs) { \
     NC = NextTab(C, TabSize); \
     if (StateMap) StateMap[i] = hsState(State);\
-    if (B) MoveChar(B, C - Pos, Width, ' ', Color, NC - C);\
+    if (B) MoveChar(B, C - Pos, Width, ' ', HILIT_CLRD(), NC - C);\
     if (BFI(BF, BFI_ShowTabs)) ColorChar();\
     i++,len--,p++;\
     C = NC;\
     continue;\
     }
 
-#define CK_MAXLEN 32
+#define CK_MAXLEN 64
+
+inline bool isZeroArray(int* Count, size_t len)
+{
+    for (size_t i = 0; i < len; ++i)
+        if (Count[i] != 0)
+            return 0;
+    return 1;
+}
+
+#define TEST_ZERO isZeroArray(Count, sizeof(Count)/sizeof(Count[0]))
 
 typedef struct {
     int TotalCount;
@@ -157,6 +202,7 @@ struct HTrans {
     long matchFlags;
     int nextState;
     int color;
+    RxNode *regexp;
     
     void InitTrans();
 };
@@ -214,7 +260,9 @@ EColorize *FindColorizer(const char *AName);
 SyntaxProc GetHilitProc(int id);
 
 int IsState(hsState *Buf, hsState State, int Len);
-int LookAt(EBuffer *B, int Row, unsigned int Pos, const char *What, hsState State, int NoWord = 1);
+int LookAt(EBuffer *B, int Row, unsigned int Pos, const char *What, hsState State, int NoWord = 1, int CaseInsensitive = 0);
+inline int LookAtNoCase(EBuffer *B, int Row, unsigned int Pos, const char *What, hsState State, int NoWord = 1)
+{ return LookAt(B, Row, Pos, What, State, NoWord, 1); }
 
 #endif
 
