@@ -18,9 +18,16 @@
 #define HISTORY_NAME "fte.his"
 #endif
 
+typedef struct {
+    char *Name;
+    int Row,Col;
+} HBookmark;
+
 typedef struct {
     char *FileName;
     int Row, Col;
+    HBookmark **Books;
+    int BookCount;
 } FPosHistory;
 
 #define MAX_INPUT_HIST 128
@@ -33,7 +40,7 @@ typedef struct {
 
 extern char HistoryFileName[256];
 
-
+void ClearHistory();
 int SaveHistory(char *FileName);
 int LoadHistory(char *FileName);
 
@@ -44,6 +51,17 @@ int AddInputHistory(int Id, char *String);
 int CountInputHistory(int Id);
 int GetInputHistory(int Id, char *String, int maxlen, int Nth);
 
+// some platforms don't know about EBuffer yet
+class EBuffer;
+/*
+ * Get bookmarks for given Buffer (file) from history.
+ */
+int RetrieveBookmarks(EBuffer *buf);
+/*
+ * Store given Buffer's bookmarks to history.
+ */
+int StoreBookmarks(EBuffer *buf);
+
 /* history values */
 #define HIST_DEFAULT    0
 #define HIST_PATH       1
@@ -57,6 +75,9 @@ int GetInputHistory(int Id, char *String, int maxlen, int Nth);
 #define HIST_REGEXP     9
 #define HIST_TRANS     10
 #define HIST_TAGFILES  11
+#define HIST_CVS       12
+#define HIST_CVSDIFF   13
+#define HIST_CVSCOMMIT 14
 
 #endif
 
