@@ -10,25 +10,13 @@
  */
 
 
-#include <stdio.h>
-#include <stdarg.h>
-#include <string.h>
-#include <stdlib.h>
-#include <assert.h>
-#include <unistd.h>
-#include <fcntl.h>
-#include <sys/types.h>
-#include <sys/time.h>
-#include <sys/wait.h>
-#include <signal.h>
-#include <ctype.h>
-#if defined(AIX)
-#include <sys/select.h>
-#endif
-#include <X11/Xatom.h>
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
+#include <stdio.h>
+#include <string.h>
+#include <stdlib.h>
+#include <ctype.h>
 
 #include "con_i18n.h"
 
@@ -52,7 +40,7 @@
  * Quite a complex function to convert normal keys
  * to dead-keys and remapped keys
  */
-static int I18NKeyAnalyze(XKeyEvent * keyEvent, KeySym * key, /*fold00*/
+static int i18n_key_analyze(XKeyEvent * keyEvent, KeySym * key, /*FOLD00*/
 			  char *keyName, int nbytes)
 {
     static long prev_state = 0, local_switch = 0,
@@ -177,8 +165,8 @@ static int I18NKeyAnalyze(XKeyEvent * keyEvent, KeySym * key, /*fold00*/
  * but as I only need ISO-8859 encoding support,
  * I don't care about this (for now).
  */
-static int I18NKeyAnalyze(XKeyEvent * keyEvent, KeySym * key, /*fold00*/
-			  char *keyName, int nbytes)
+static int i18n_key_analyze(XKeyEvent * /*keyEvent*/, KeySym * key, /*FOLD00*/
+			    char *keyName, int nbytes)
 {
     KeySym t = (unsigned char) keyName[0];
 
@@ -216,14 +204,12 @@ static int I18NKeyAnalyze(XKeyEvent * keyEvent, KeySym * key, /*fold00*/
  * is using Xt Toolkit some things have to be made
  * different
  */
-XIC I18NInit(Display * display, Window win, unsigned long *mask) /*fold00*/
+i18n_context_t* i18n_open(Display * display, Window win, unsigned long *mask) /*FOLD00*/
 {
-    XIC xic = (XIC) NULL;
+    i18n_context_t* ctx = (i18n_context_t*) malloc(sizeof(i18n_context_t));
+    memset(ctx, 0, sizeof(i18n_context_t));
 
 #if XlibSpecificationRelease >= 6
-    XIM xim = (XIM) NULL;
-    XIMStyles *xim_styles;
-    XIMStyle input_style = 0;
     char *s, tmp[256];
     int found = False;
 
@@ -231,36 +217,40 @@ XIC I18NInit(Display * display, Window win, unsigned long *mask) /*fold00*/
 
     /* Locale setting taken from XtSetLanguageProc */
     if (!(s = setlocale(LC_ALL, "")))
-	fprintf(stderr, "I18N: Locale not supported by C library, "
+	fprintf(stderr, "I18N warning: Locale not supported by C library, "
 		"locale unchanged!\n");
 
     if (!XSupportsLocale()) {
-	fprintf(stderr, "I18N: Locale not supported by Xlib, "
+	fprintf(stderr, "I18N warning: Locale not supported by Xlib, "
 		"locale set to C!\n");
 	s = setlocale(LC_ALL, "C");
     }
     if (!XSetLocaleModifiers(""))
-	fprintf(stderr, "I18N: X locale modifiers not supported, "
+	fprintf(stderr, "I18N warning: X locale modifiers not supported, "
 		"using default\n");
 
-    if ((xim = XOpenIM(display, NULL, NULL, NULL)) == NULL) {
-	fprintf(stderr, "I18N: Failed to open input method\n");
-	return xic;
+    ctx->xim = XOpenIM(display, NULL, NULL, NULL);
+    if (ctx->xim == NULL) {
+        // there are languages without Input Methods ????
+	fprintf(stderr, "I18N warning: Input method not specified\n");
+	i18n_destroy(ctx);
+	return NULL;
     }
-    if (XGetIMValues(xim, XNQueryInputStyle, &xim_styles, NULL)
-	|| (xim_styles == NULL)) {
-	fprintf(stderr, "I18N: Input method doesn't support any style\n");
-	XCloseIM(xim);
-	return xic;
+
+    if (XGetIMValues(ctx->xim, XNQueryInputStyle, &ctx->xim_styles, NULL)
+	|| ctx->xim_styles == NULL) {
+	fprintf(stderr, "I18N error: Input method doesn't support any style\n");
+        i18n_destroy(ctx);
+	return NULL;
     }
 
     /*
      * This is some kind of debugging message to inform user
      * that something is wrong with his system
      */
-    if ((s != NULL) && (strstr(s, XLocaleOfIM(xim)) == NULL))
+    if (s != NULL && (strstr(s, XLocaleOfIM(ctx->xim)) == NULL))
 	fprintf(stderr, "I18N warning: System locale: \"%s\" differs from "
-		"IM locale: \"%s\"...\n", s, XLocaleOfIM(xim));
+		"IM locale: \"%s\"...\n", s, XLocaleOfIM(ctx->xim));
 
     /*
      * This part is cut&paste from other sources
@@ -290,63 +280,84 @@ XIC I18NInit(Display * display, Window win, unsigned long *mask) /*fold00*/
 	*end = '\0';
 
 	if (!strcmp(s, "OverTheSpot"))
-	    input_style = (XIMPreeditPosition | XIMStatusArea);
+	    ctx->input_style = (XIMPreeditPosition | XIMStatusArea);
 	else if (!strcmp(s, "OffTheSpot"))
-	    input_style = (XIMPreeditArea | XIMStatusArea);
+	    ctx->input_style = (XIMPreeditArea | XIMStatusArea);
 	else if (!strcmp(s, "Root"))
-	    input_style = (XIMPreeditNothing | XIMStatusNothing);
+	    ctx->input_style = (XIMPreeditNothing | XIMStatusNothing);
 
-	for (i = 0; (unsigned short) i < xim_styles->count_styles; i++)
-	    if (input_style == xim_styles->supported_styles[i]) {
+	for (i = 0; (unsigned short) i < ctx->xim_styles->count_styles; i++)
+	    if (ctx->input_style == ctx->xim_styles->supported_styles[i])
+	    //if (ctx->xim_styles->supported_styles[i] & (XIMPreeditNothing | XIMPreeditNone) &&
+            //    ctx->xim_styles->supported_styles[i] & (XIMStatusNothing  | XIMStatusNone))
+	    {
 		found = True;
+		/* do not modify this!
+		 * unless consulted with kabi@users.sf.net
+		 */
+		//ctx->input_style = ctx->xim_styles->supported_styles[i];
 		break;
 	    }
 	s = ns;
     }
-    XFree(xim_styles);
+    XFree(ctx->xim_styles);
 
     if (!found) {
-	fprintf(stderr, "I18N: Input method doesn't support my preedit type\n");
-	XCloseIM(xim);
-	return xic;
+	fprintf(stderr, "I18N error: Input method doesn't support my "
+		"preedit type\n");
+        i18n_destroy(ctx);
+	return NULL;
     }
     /* This program only understand the Root preedit_style yet */
-    if (input_style != (XIMPreeditNothing | XIMStatusNothing)) {
-	fprintf(stderr, "I18N: This program only supports the "
+    if (ctx->input_style != (XIMPreeditNothing | XIMStatusNothing)) {
+	fprintf(stderr, "I18N error: This program only supports the "
 		"'Root' preedit type\n");
-	XCloseIM(xim);
-	return xic;
+        i18n_destroy(ctx);
+	return NULL;
     }
-    xic = XCreateIC(xim, XNInputStyle, input_style,
-		    XNClientWindow, win,
-		    XNFocusWindow, win,
-		    NULL);
-    if (xic == NULL) {
-	fprintf(stderr, "I18N: Failed to create input context\n");
-	XCloseIM(xim);
-    } else if (XGetICValues(xic, XNFilterEvents, mask, NULL))
-	fprintf(stderr, "I18N: Can't get Event Mask\n");
+    ctx->xic = XCreateIC(ctx->xim, XNInputStyle, ctx->input_style,
+			 XNClientWindow, win,
+			 XNFocusWindow, win,
+			 NULL);
+    if (ctx->xic == NULL) {
+	fprintf(stderr, "I18N error: Failed to create input context\n");
+        i18n_destroy(ctx);
+    } else if (XGetICValues(ctx->xic, XNFilterEvents, mask, NULL))
+	fprintf(stderr, "I18N error: Can't get Event Mask\n");
 #else
     *mask = 0;
 #endif
-    return xic;
+    return ctx;
 }
  /*FOLD00*/
 
-void I18NFocusIn(XIC xic) /*fold00*/
+void i18n_destroy(i18n_context_t* ctx)
+{
+    if (ctx)
+    {
+        if (ctx->xic)
+	    XDestroyIC(ctx->xic);
+	if (ctx->xim)
+	    XCloseIM(ctx->xim);
+        free(ctx);
+    }
+}
+
+
+void i18n_focus_in(i18n_context_t* ctx) /*FOLD00*/
 {
 #if XlibSpecificationRelease >= 6
-    if (xic != NULL)
-	XSetICFocus(xic);
+    if (ctx->xic != NULL)
+	XSetICFocus(ctx->xic);
 #endif
 }
  /*FOLD00*/
 
-void I18NFocusOut(XIC xic) /*fold00*/
+void i18n_focus_out(i18n_context_t* ctx) /*FOLD00*/
 {
 #if XlibSpecificationRelease >= 6
-    if (xic != NULL)
-	XUnsetICFocus(xic);
+    if (ctx->xic != NULL)
+	XUnsetICFocus(ctx->xic);
 #endif
 }
  /*FOLD00*/
@@ -354,7 +365,7 @@ void I18NFocusOut(XIC xic) /*fold00*/
 /*
  * Lookup correct keysymbol from keymap event
  */
-int I18NLookupString(XKeyEvent * keyEvent, char *keyName, int keySize, /*fold00*/
+int i18n_lookup_sym(XKeyEvent * keyEvent, char *keyName, int keySize, /*FOLD00*/
 		     KeySym * key, XIC xic)
 {
     static int showKeys = 0;
@@ -363,11 +374,19 @@ int I18NLookupString(XKeyEvent * keyEvent, char *keyName, int keySize, /*fold00*
 #if XlibSpecificationRelease >= 6
     if (xic != NULL) {
 	if (keyEvent->type == KeyPress) {
-	    Status status_return;
+            Status status_return;
 
-	    /* No KeyRelease events here ! */
+            /* No KeyRelease events here ! */
+#if 1
 	    nbytes = XmbLookupString(xic, keyEvent, keyName, keySize,
-				     key, &status_return);
+                                     key, &status_return);
+#else
+            wchar_t wk;
+            nbytes = XwcLookupString(xic, keyEvent, &wk, 1, key, &status_return);
+            printf("code=%0X\n", wk);
+            keySize = 1;
+            keyName[0] = wk & 0xFF;
+#endif
 	}
     } else
 #endif
@@ -393,6 +412,6 @@ int I18NLookupString(XKeyEvent * keyEvent, char *keyName, int keySize, /*fold00*
 	&& (keyEvent->type == KeyPress))
 	showKeys = !showKeys;
 
-    return I18NKeyAnalyze(keyEvent, key, keyName, nbytes);
+    return i18n_key_analyze(keyEvent, key, keyName, nbytes);
 }
  /*FOLD00*/
