@@ -32,26 +32,31 @@
  *                      mainloop when console returns empty action..
  *
  */
-//#define WIN32_LEAN_AND_MEAN 1
-#include "windows.h"
+#define WIN32_LEAN_AND_MEAN
+#include <windows.h>
 #include <process.h>
+#include <signal.h>
+
+#include <wincon.h>
 
 #include <stdio.h>
 #include "sysdep.h"
 #include "console.h"
 #include "gui.h"
+#include "s_files.h"
+#include "c_config.h"
 
 #define True    1
 #define False   0
 
 
-static Initialized = 0;
-static MousePresent = 0;
+static int Initialized = 0;
+static int MousePresent = 0;
 static int CursorVisible = 1; /* 1 means visible */
 static int MouseVisible = 0; /* 0 means hidden */
 static TEvent MouseEv = { evNone };
 static TEvent EventBuf = { evNone };
-static TEventMask EventMask;
+//static TEventMask EventMask;
 
 static HANDLE ConOut;
 static HANDLE ConIn;
@@ -62,6 +67,9 @@ static int LastMouseX = 0;
 static int      LastMouseY = 0;
 //static int      isWin95 = 0;
 
+static char winTitle[256] = "FTE";
+static char winSTitle[256] = "FTE";
+
 int codepage;
 
 #define dbm(x)          //printf(x), Sleep(3000)
@@ -283,6 +291,9 @@ int ReadConsoleEvent(TEvent *E) /*FOLD00*/
 
         //** Distill FTE flags from the NT flags. This fails for some keys
         //** because NT has an oddity with enhanced keys (Alt-Grey-+ etc).
+
+        // from chaac: Please do not toutch RIGHT_ALT_PRESSED handling,
+        // in some keyboard ALT-GR is used for special characters.
         flags = inp.Event.KeyEvent.dwControlKeyState;
         if (flags & (/*RIGHT_ALT_PRESSED |*/ LEFT_ALT_PRESSED)) flg |= kfAlt;
         if (flags & (RIGHT_CTRL_PRESSED | LEFT_CTRL_PRESSED)) flg |= kfCtrl;
@@ -300,15 +311,34 @@ int ReadConsoleEvent(TEvent *E) /*FOLD00*/
             inp.Event.KeyEvent.uChar.AsciiChar,
             inp.Event.KeyEvent.uChar.AsciiChar);
 #endif
+        Ch = 0;
+
+        // handle special case when user with scandinavian keyboard presses
+        // alt-gr + special key and then spacebar
+        if (inp.Event.KeyEvent.bKeyDown) {
+            if ((inp.Event.KeyEvent.wVirtualKeyCode == 0x20) &&
+                (inp.Event.KeyEvent.wVirtualScanCode == 0x39))
+            {
+                switch(inp.Event.KeyEvent.uChar.AsciiChar)
+                {
+                case '~': Ch = '~'; break;
+                case '^': Ch = '^'; break;
+                case '`': Ch = '`'; break;
+                case '': Ch = ''; break;
+                }
+            }
+        }
 
         //** Translate VK codes to FTE codes,
-        Ch = 0;
-        for (I = 0; I < sizeof(VirtTab)/sizeof(VirtTab[0]); I++)
-            if (VirtTab[I].VirtCode == inp.Event.KeyEvent.wVirtualKeyCode)
-            {
-                Ch = VirtTab[I].KeyCode;
-                break;
-            }
+        if (Ch == 0)
+        {
+            for (I = 0; I < sizeof(VirtTab)/sizeof(VirtTab[0]); I++)
+                if (VirtTab[I].VirtCode == inp.Event.KeyEvent.wVirtualKeyCode)
+                {
+                    Ch = VirtTab[I].KeyCode;
+                    break;
+                }
+        }
 
         //** Not a virtual key-> do charscan translation, if needed;
         if(Ch == 0)
@@ -421,6 +451,7 @@ int ConDone(void) { /*FOLD00*/
 int ConSuspend(void) { /*FOLD00*/
     SetConsoleActiveScreenBuffer(ConOut);
     SetConsoleMode(ConIn, OldConsoleMode);
+
     return 0;
 }
 
@@ -756,6 +787,10 @@ GUI::GUI(int &argc, char **argv, int XSize, int YSize) { /*FOLD00*/
 
 GUI::~GUI() { /*FOLD00*/
     RestoreScreen();
+
+    if (SavedScreen)
+        free(SavedScreen);
+
     ::ConDone();
     gui = 0;
 }
@@ -785,9 +820,12 @@ int GUI::ShowEntryScreen() { /*FOLD00*/
 }
 
 char ConGetDrawChar(int index) { /*FOLD00*/
-    static char tab[] = "ڿĳô\x1A\x1B\x1A";
+    static const char *tab=NULL;
 
-    assert(index >= 0 && index < strlen(tab));
+    if (!tab) {
+        tab=GetGUICharacters ("WindowsNT","ڿĳô\x1A\x1B\x1A");
+    }
+    assert(index >= 0 && index < (int)strlen(tab));
 
     return tab[index];
 }
@@ -805,8 +843,16 @@ int GUI::RunProgram(int mode, char *Command) { /*FOLD00*/
                          "COMSPEC"
                         );
 
+    // we don't want ctrl-c or ctrl-break to exit our editor...
+    signal(SIGBREAK, SIG_IGN);
+    signal(SIGINT, SIG_IGN);
+
     rc = system(Command);
 
+    // restore handlers back to default handlers
+    signal(SIGBREAK, SIG_DFL);
+    signal(SIGINT, SIG_DFL);
+
     ConContinue();
     ConShowMouse();
     ConQuerySize(&W1, &H1);
@@ -819,12 +865,31 @@ int GUI::RunProgram(int mode, char *Command) { /*FOLD00*/
 }
 
 int ConSetTitle(char *Title, char *STitle) { /*FOLD00*/
+    char buf[sizeof(winTitle)] = {0};
+    JustFileName(Title, buf);
+    if (buf[0] == '\0') // if there is no filename, try the directory name.
+        JustLastDirectory(Title, buf);
+
+    strncpy(winTitle, "FTE - ", sizeof(winTitle) - 1);
+    if (buf[0] != 0) // if there is a file/dir name, stick it in here.
+    {
+        strncat(winTitle, buf, sizeof(winTitle) - 1 - strlen(winTitle));
+        strncat(winTitle, " - ", sizeof(winTitle) - 1 - strlen(winTitle));
+    }
+    strncat(winTitle, Title, sizeof(winTitle) - 1 - strlen(winTitle));
+    winTitle[sizeof(winTitle) - 1] = 0;
+    strncpy(winSTitle, STitle, sizeof(winSTitle) - 1);
+    winSTitle[sizeof(winSTitle) - 1] = 0;
+    SetConsoleTitle (winTitle);
+
     return 0;
 }
 
 int ConGetTitle(char *Title, int MaxLen, char *STitle, int SMaxLen) { /*FOLD00*/
-    strcpy(Title, "FTE");
-    strcpy(STitle, "FTE");
+    strncpy(Title, winTitle, MaxLen);
+    Title[MaxLen - 1] = 0;
+    strncpy(STitle, winSTitle, SMaxLen);
+    STitle[SMaxLen - 1] = 0;
     return 0;
 }
 
@@ -1479,7 +1544,7 @@ static GPipe Pipes[MAX_PIPES] = {
 };
 
 static int CreatePipeChild(HANDLE &child, HANDLE &hPipe, char *Command) {
-    static int PCount = 0;
+    //static int PCount = 0;
     int arglen = 0;
     HANDLE hChildPipe;
     BOOL rc;
