@@ -68,8 +68,8 @@ static long MouseAutoDelay = 400;
 static long MouseAutoRepeat = 5;
 static long MouseMultiClick = 300;
 
-static Initialized = 0;
-static MousePresent = 0;
+static int Initialized = 0;
+static int MousePresent = 0;
 static int CursorVisible = 1; /* 1 means visible */
 static int MouseVisible = 0; /* 0 means hidden */
 static TEvent MouseEv = { evNone };
@@ -85,7 +85,7 @@ static void DrawCursor(int Show) {
     if (Show == 1)
         vci.attr = 1;
     else
-        vci.attr = -1;
+        vci.attr = (SHORT)-1;
     VioSetCurType(&vci, 0);
 }
 
@@ -794,6 +794,10 @@ GUI::GUI(int &argc, char **argv, int XSize, int YSize) {
 
 GUI::~GUI() {
     RestoreScreen();
+
+    if (SavedScreen)
+        free(SavedScreen);
+
     ::ConDone();
     gui = 0;
 }
@@ -831,7 +835,7 @@ static int CreatePipeChild(PID &pid, HPIPE &hfPipe, char *Command) {
     ULONG ulAction;
     //ULONG ulNew;
     HPIPE hfChildPipe;
-    HFILE hfNewStdOut = -1, hfNewStdErr = -1;
+    HFILE hfNewStdOut = (HFILE)-1, hfNewStdErr = (HFILE)-1;
     HFILE hfStdOut = 1, hfStdErr = 2;
     int rc;
 
@@ -1002,7 +1006,10 @@ int GUI::OpenPipe(char *Command, EModel *notify) {
             Pipes[i].notify = notify;
             Pipes[i].DoTerm = 0;
             if ((Pipes[i].buffer = (char *)malloc(PIPE_BUFLEN)) == 0)
+            {
+                free(Pipes[i].Command);
                 return -1;
+            }
 
             if (0 != DosCreateMutexSem(0, &Pipes[i].Access, 0, 0)) {
                 free(Pipes[i].Command);
@@ -1210,7 +1217,7 @@ char ConGetDrawChar(int index) {
     static char tab[] =  "ڿĳô\x1A\x1B\x1A";
     static char tab7[] = "++++-|+++++\x1A.-++#+\x1B\x1A";
 
-    assert(index >= 0 && index < strlen(tab));
+    assert(index >= 0 && index < (int)strlen(tab));
 
     if (SevenBit)
         return tab7[index];
