@@ -12,18 +12,19 @@
 #include <assert.h>
 #include <stdarg.h>
 #ifdef WINNT
+#include <winsock.h>
 #define NO_PIPES
 #define NO_SIGNALS
 #else
 #include <unistd.h>
 #include <sys/wait.h>
+#include <sys/time.h>
 #endif
 
 #include <time.h>
 #include <stdio.h>
 #include <fcntl.h>
 #include <sys/types.h>
-#include <sys/time.h>
 #include <stdlib.h>
 #include <signal.h>
 #if defined(AIX)
@@ -35,21 +36,28 @@
 #include <X11/Xutil.h>
 #include <X11/keysym.h>
 #include <X11/Xos.h>
+#ifdef USE_XTINIT
+#include <X11/Intrinsic.h>
+#endif
 #ifdef HPUX
 #include </usr/include/X11R5/X11/HPkeysym.h>
 #endif
+#include "sysdep.h"
+#include "c_config.h"
 #include "console.h"
 #include "gui.h"
 
 #include "con_i18n.h"
+#include "s_files.h"
+#include "s_util.h"
 
-XIC xic = NULL;
+i18n_context_t* i18n_ctx = NULL;
 
 #ifdef WINHCLX
 #include <X11/XlibXtra.h>    /* HCL - HCLXlibInit */
 #endif
 
-#ifdef HPUX
+#ifdef CAST_FD_SET_INT
 #define FD_SET_CAST() (int *)
 #else
 #define FD_SET_CAST()
@@ -74,9 +82,9 @@ static GPipe Pipes[MAX_PIPES] = {
     { 0 },
 };
 
-static long MouseAutoDelay = 40;
-static long MouseAutoRepeat = 200;
-static long MouseMultiClick = 300;
+static const long MouseAutoDelay = 40;
+static const long MouseAutoRepeat = 200;
+static const long MouseMultiClick = 300;
 
 static int setUserPosition = 0;
 static int initX = 0, initY = 0;
@@ -97,7 +105,7 @@ static Window win;
 static Atom selection_buffer;
 static XSizeHints sizeHints;
 // program now contains both modes if available
-// some older Xserver doesn't like XmbDraw...
+// some older Xservers don't like XmbDraw...
 static XFontStruct *fontStruct;
 #ifdef USE_XMB
 static int useXMB = 1; // default is yes
@@ -109,11 +117,11 @@ static int useXMB = 0;
 static int FontCX, FontCY;
 static XColor Colors[16];
 static GC GCs[256];
-static int rc;
+//static int rc;
 static char winTitle[256] = "FTE";
 static char winSTitle[256] = "FTE";
 
-static char *CurSelectionData = 0;
+static unsigned char *CurSelectionData = 0;
 static int CurSelectionLen = 0;
 static int CurSelectionOwn = 0;
 static Time now;
@@ -228,7 +236,8 @@ static int InitXColors() {
     return 0;
 }
 
-static int InitXGCs() {
+static int InitXGCs()
+{
     unsigned int i;
     unsigned long mask = GCForeground | GCBackground;
     XGCValues gcv;
@@ -247,20 +256,53 @@ static int InitXGCs() {
     return 0;
 }
 
+#ifdef USE_XMB
+static void try_fontset_load(const char *fs)
+{
+    char *def = NULL;
+    char **miss = NULL;
+    int nMiss = 0;
+
+    if (fontSet)
+	return;
+
+    if (!fs || !*fs)
+	return;
+
+    fontSet = XCreateFontSet(display, fs, &miss, &nMiss,
+			     &def);
+
+    if (fontSet == NULL) {
+	fprintf(stderr, "XFTE Warning: unable to open font \"%s\":\n"
+		" Missing count: %d\n", fs, nMiss);
+	for(int i = 0; i < nMiss; i++)
+	    fprintf(stderr, "  %s\n", miss[i]);
+	if (def != NULL)
+	    fprintf(stderr, " def_ret: %s\n", def);
+    }
+}
+#endif
+
 static int InitXFonts(void)
 {
     char *fs;
 
-
     fs = getenv("VIOFONT");
     if (fs == NULL && WindowFont[0] != 0)
         fs = WindowFont;
 
     if (!useXMB) {
+
         fontStruct = NULL;
 
-        if (fs != NULL)
-            fontStruct = XLoadQueryFont(display, fs);
+	if (fs != NULL) {
+	    char *s = 0;
+
+	    s = strchr(fs, ',');
+	    if (s != NULL)
+		*s = 0;
+	    fontStruct = XLoadQueryFont(display, fs);
+	}
         if (fontStruct == NULL)
             fontStruct = XLoadQueryFont(display, "8x13");
         if (fontStruct == NULL)
@@ -272,37 +314,14 @@ static int InitXFonts(void)
     }
 #ifdef USE_XMB
     else {
-        const char *def = " ";
-        char **miss;
-        int nMiss;
+	try_fontset_load(getenv("VIOFONT"));
+	try_fontset_load(WindowFont);
+	try_fontset_load("-misc-*-r-normal-*");
+	try_fontset_load("*fixed*");
 
-        // test font fixed font:
-        // fs="*-fixed-bold-*-15-*";
-        if (fs != NULL)
-            fontSet = XCreateFontSet(display, fs, &miss, &nMiss,
-                                     (char **) &def);
-        // printf("FS %p %p %d %s %s\n", fontSet, fs, WindowFont);
+	if (fontSet == NULL)
+	    return -1;
 
-        // try any fixed localized font */
-        if (fontSet == NULL)
-            fontSet = XCreateFontSet(display, "*fixed*", &miss, &nMiss,
-                                     (char **) &def);
-
-        // try plain fixed font
-        if (fontSet == NULL)
-            fontSet = XCreateFontSet(display, "fixed", &miss, &nMiss,
-                                     (char **) &def);
-
-        if (fontSet == NULL) {
-            fprintf(stderr, "Could not get fontset\n"
-                    "Missing count: %d\n", nMiss);
-            for(int i = 0; i < nMiss; i++)
-                fprintf(stderr, "  %s\n", miss[i]);
-            if (def != NULL)
-                fprintf(stderr, "def_ret: %s\n", def);
-        }
-        if (fontSet == NULL)
-            return -1;
         XFontSetExtents *xE = XExtentsOfFontSet(fontSet);
 
         FontCX = xE->max_logical_extent.width;
@@ -315,27 +334,36 @@ static int InitXFonts(void)
     return 0;
 }
 
-static int SetupXWindow() {
+static int SetupXWindow(int argc, char **argv)
+{
     unsigned long mask;
-    char *ds;
     XSetWindowAttributes setWindowAttributes;
 
-    if ((ds = getenv("DISPLAY")) == NULL)
-        DieError(1, "$DISPLAY not set?");
-
 #ifdef WINHCLX
-    HCLXlibInit(); /* HCL - Initialize the X DLL        */
+    HCLXlibInit(); /* HCL - Initialize the X DLL */
 #endif
 
+#ifdef USE_XTINIT
+    XtAppContext  	app_context;
+    XtToolkitInitialize();
+    app_context = XtCreateApplicationContext();
+    if (( display = XtOpenDisplay(app_context, NULL, argv[0], "xfte",
+                            NULL, 0, &argc, argv)) == NULL)
+       DieError(1, "%s:  Can't open display\n", argv[0]);
+#else
+    char *ds;
+    if ((ds = getenv("DISPLAY")) == NULL)
+	DieError(1, "$DISPLAY not set?");
     if ((display = XOpenDisplay(ds)) == NULL)
-        DieError(1, "Could not open display: %s!", ds);
+	DieError(1, "XFTE Fatal: could not open display: %s!", ds);
+#endif
 
     colormap = DefaultColormap(display, DefaultScreen(display));
 
     setWindowAttributes.bit_gravity =
         sizeHints.win_gravity = NorthWestGravity;
 
-    // this is correct behauvier
+    // this is correct behavior
     if (initX < 0)
         initX = DisplayWidth(display, DefaultScreen(display)) + initX;
     if (initY < 0)
@@ -350,10 +378,10 @@ static int SetupXWindow() {
                         CopyFromParent, InputOutput, CopyFromParent,
                         CWBitGravity, &setWindowAttributes);
 
-    xic = I18NInit(display, win, &mask);
+    i18n_ctx = i18n_open(display, win, &mask);
 
     if (InitXFonts() != 0)
-        DieError(1, "Could not open any font!");
+	DieError(1, "XFTE Fatal: could not open any font!");
 
     /* >KeyReleaseMask shouldn't be set for correct key mapping */
     /* we set it anyway, but not pass to XmbLookupString -- mark */
@@ -382,8 +410,10 @@ static int SetupXWindow() {
         sizeHints.flags |= USPosition;
 
     XClassHint classHints;
-    classHints.res_name = (char *)"fte";
-    classHints.res_class = (char *)"Fte";
+    static char res_name[] = "fte";
+    static char res_class[] = "Fte";
+    classHints.res_name = res_name;
+    classHints.res_class = res_class;
     XSetClassHint(display, win, &classHints);
 
     XSetStandardProperties(display, win, winTitle, winTitle, 0, NULL, 0, 0);
@@ -405,7 +435,6 @@ int ConInit(int XSize, int YSize) {
     if (YSize != -1)
         ScreenRows = YSize;
     if (AllocBuffer() == -1) return -1;
-    if (SetupXWindow() == -1) return -1;
 #ifndef NO_SIGNALS
     signal(SIGALRM, SIG_IGN);
     signal(SIGPIPE, SIG_IGN);
@@ -414,8 +443,7 @@ int ConInit(int XSize, int YSize) {
 }
 
 int ConDone(void) {
-    XDestroyWindow(display, win);
-    XCloseDisplay(display);
+    free(ScreenBuffer);
     return 0;
 }
 
@@ -434,7 +462,18 @@ int ConClear(void) {
 }
 
 int ConSetTitle(char *Title, char *STitle) {
-    strncpy(winTitle, Title, sizeof(winTitle) - 1);
+    char buf[sizeof(winTitle)] = {0};
+    JustFileName(Title, buf);
+    if (buf[0] == '\0') // if there is no filename, try the directory name.
+        JustLastDirectory(Title, buf);
+
+    strncpy(winTitle, "FTE - ", sizeof(winTitle) - 1);
+    if (buf[0] != 0) // if there is a file/dir name, stick it in here.
+    {
+        strncat(winTitle, buf, sizeof(winTitle) - 1 - strlen(winTitle));
+        strncat(winTitle, " - ", sizeof(winTitle) - 1 - strlen(winTitle));
+    }
+    strncat(winTitle, Title, sizeof(winTitle) - 1 - strlen(winTitle));
     winTitle[sizeof(winTitle) - 1] = 0;
     strncpy(winSTitle, STitle, sizeof(winSTitle) - 1);
     winSTitle[sizeof(winSTitle) - 1] = 0;
@@ -484,17 +523,18 @@ int ConPutBox(int X, int Y, int W, int H, PCell Cell) {
 
 
     if (X >= (int) ScreenCols || Y >= (int) ScreenRows ||
-        X + W > (int) ScreenCols || Y + H > (int) ScreenRows)
-    {
+        X + W > (int) ScreenCols || Y + H > (int) ScreenRows) {
         //fprintf(stderr, "%d %d  %d %d %d %d\n", ScreenCols, ScreenRows, X, Y, W, H);
         return -1;
     }
+    //XClearArea(display, win, X, Y, W * FontCX, H * FontCY, False);
 
+    //fprintf(stderr, "%d %d  %d %d %d %d\n", ScreenCols, ScreenRows, X, Y, W, H);
     for (i = 0; i < (unsigned int)H; i++) {
         len = W;
         p = CursorXYPos(X, Y + i);
         ps = (unsigned char *) Cell;
-        x = X;
+	x = X;
         while (len > 0) {
             if (!Refresh) {
                 c = CursorXYPos(x, Y + i);
@@ -520,12 +560,11 @@ int ConPutBox(int X, int Y, int W, int H, PCell Cell) {
             p = ps;
             l = 1;
             temp[0] = *ps++; attr = *ps++;
-            while ((l < len) && ((unsigned char) (ps[1]) == attr))
-            {
+            while ((l < len) && ((unsigned char) (ps[1]) == attr)) {
                 temp[l++] = *ps++;
                 ps++;
-            }
-            if (!useXMB)
+	    }
+	    if (!useXMB)
                 XDrawImageString(display, win, GCs[((unsigned)attr) & 0xFF],
                                  x * FontCX, fontStruct->max_bounds.ascent +
                                  (Y + i) * FontCY,
@@ -537,11 +576,17 @@ int ConPutBox(int X, int Y, int W, int H, PCell Cell) {
                                    x * FontCX, FontCYD + (Y + i) * FontCY,
                                    (char *)temp, l);
 #endif
-            //{int i; for(i=0; i<l; i++) printf("%c", temp[i]); printf("\n"); }
-            x += l;
+	    //temp[l] = 0; printf("%s\n", temp);
             len -= l;
-        }
-        p = CursorXYPos(X, Y + i);
+            x += l;
+	}
+/*	if (x < ScreenCols - 1) {
+	    printf("XX %d   %d   %d\n", X, x, W);
+	    XFillRectangle(display, win, GCs[15 * 16 + 7],
+			   x * FontCX, (Y + i) * FontCY,
+			   (ScreenCols - x - 1) * FontCX, FontCY);
+	}
+*/        p = CursorXYPos(X, Y + i);
         memcpy(p, Cell, W * 2);
         if (i + Y == CursorY)
             DrawCursor(1);
@@ -572,7 +617,8 @@ int ConSetBox(int X, int Y, int W, int H, TCell Cell) {
     TDrawBuffer B;
     int i;
 
-    for (i = 0; i < W; i++) B[i] = Cell;
+    for (i = 0; i < W; i++)
+	B[i] = Cell;
     ConPutLine(X, Y, W, H, B);
     return 0;
 }
@@ -584,18 +630,18 @@ int ConScroll(int Way, int X, int Y, int W, int H, TAttr Fill, int Count) {
     MoveCh(&Cell, ' ', Fill, 1);
     DrawCursor(0);
     if (Way == csUp) {
-        XCopyArea(display, win, win, GCs[0],
-                  X * FontCX,
+	XCopyArea(display, win, win, GCs[0],
+		  X * FontCX,
                   (Y + Count) * FontCY,
                   W * FontCX,
                   (H - Count) * FontCY,
-                  X * FontCX,
-                  Y * FontCY
-                 );
-        for (l = 0; l < H - Count; l++) {
-            memcpy(CursorXYPos(X, Y + l), CursorXYPos(X, Y + l + Count), 2 * W);
-        }
-        if (ConSetBox(X, Y + H - Count, W, Count, Cell) == -1) return -1;
+		  X * FontCX,
+                  Y * FontCY);
+	for (l = 0; l < H - Count; l++)
+	    memcpy(CursorXYPos(X, Y + l), CursorXYPos(X, Y + l + Count), 2 * W);
+
+	if (ConSetBox(X, Y + l, W, Count, Cell) == -1)
+	    return -1;
     } else if (Way == csDown) {
         XCopyArea(display, win, win, GCs[0],
                   X * FontCX,
@@ -603,12 +649,12 @@ int ConScroll(int Way, int X, int Y, int W, int H, TAttr Fill, int Count) {
                   W * FontCX,
                   (H - Count) * FontCY,
                   X * FontCX,
-                  (Y + Count)* FontCY
-                 );
-        for (l = H - 1; l >= Count; l--) {
+                  (Y + Count) * FontCY);
+	for (l = H - 1; l >= Count; l--)
             memcpy(CursorXYPos(X, Y + l), CursorXYPos(X, Y + l - Count), 2 * W);
-        }
-        if (ConSetBox(X, Y, W, Count, Cell) == -1) return -1;
+
+	if (ConSetBox(X, Y, W, Count, Cell) == -1)
+	    return -1;
     }
     DrawCursor(1);
     return 0;
@@ -626,9 +672,12 @@ int ConSetSize(int X, int Y) {
         *p++ = ' ';
         *p++ = 0x07;
     }
-    MX = ScreenCols; if (X < MX) MX = X;
-    MY = ScreenRows; if (Y < MY) MY = Y;
-    if (X < MX) MX = X;
+    MX = ScreenCols;
+    if (X < MX)
+	MX = X;
+    MY = ScreenRows;
+    if (Y < MY)
+	MY = Y;
     p = NewBuffer;
     for (i = 0; i < MY; i++) {
         memcpy(p, CursorXYPos(0, i), MX * 2);
@@ -679,11 +728,11 @@ int ConHideCursor(void) {
 int ConCursorVisible(void) {
     return 1;
 }
-int ConSetCursorSize(int Start, int End) {
+int ConSetCursorSize(int /*Start*/, int /*End*/) {
     return 1;
 }
 
-int ConSetMousePos(int X, int Y) {
+int ConSetMousePos(int /*X*/, int /*Y*/) {
     return 0;
 }
 
@@ -696,10 +745,12 @@ int ConQueryMousePos(int *X, int *Y) {
 }
 
 int ConShowMouse(void) {
+    printf("Show\n");
     return 0;
 }
 
 int ConHideMouse(void) {
+    printf("Hide\n");
     return 0;
 }
 
@@ -715,10 +766,32 @@ int ConQueryMouseButtons(int *ButtonCount) {
 void UpdateWindow(int xx, int yy, int ww, int hh) {
     PCell p;
     int i;
+
+    /* show redrawn area */
+    /*
+    XFillRectangle(display, win, GCs[14], xx, yy, ww, hh);
+    XFlush(display);
+    i = XEventsQueued(display, QueuedAfterReading);
+    while (i-- > 0) {
+	XEvent e;
+	XNextEvent(display, &e);
+    }
+    // sleep(1);*/
+
     ww /= FontCX; ww += 2;
-    hh /= FontCY; hh += 2;
+    hh /= FontCY;
     xx /= FontCX;
     yy /= FontCY;
+
+    /*
+     * OK for this moment I suggest this method - it works somehow
+     * But I suppose the correct solution would meant general rewrite
+     * of some basic behavior of FTE editor
+     * THIS IS TEMPORAL FIX AND SHOULD BE SOLVED IN GENERAL WAY !
+     */
+    hh *= 3; yy -= hh; hh += hh + 2;
+    if (yy < 0)
+	yy = 0;
     if (xx + ww > (int)ScreenCols) ww = ScreenCols - xx;
     if (yy + hh > (int)ScreenRows) hh = ScreenRows - yy;
     Refresh = 1;
@@ -729,6 +802,7 @@ void UpdateWindow(int xx, int yy, int ww, int hh) {
         ConPutBox(xx, yy + i, ww, 1, p);
         p += ScreenCols;
     }
+    //fprintf(stderr, "UPDATE\tx:%3d  y:%3d  w:%3d  h:%3d\n", xx, yy, ww, hh);
     //XFlush(display);
     Refresh = 0;
 }
@@ -830,7 +904,7 @@ static struct {
     { 0,                 0 }
 };
 
-void ConvertKeyToEvent(KeySym key, KeySym key1, char *keyname, char *keyname1, int etype, int state, TEvent *Event) {
+void ConvertKeyToEvent(KeySym key, KeySym key1, char */*keyname*/, char */*keyname1*/, int etype, int state, TEvent *Event) {
     unsigned int myState = 0;
 
     Event->What = evNone;
@@ -888,12 +962,12 @@ static TEvent LastMouseEvent = { evNone };
 
 #define TM_DIFF(x,y) ((long)(((long)(x) < (long)(y)) ? ((long)(y) - (long)(x)) : ((long)(x) - (long)(y))))
 
-void ConvertClickToEvent(int type, int xx, int yy, int button, int state, TEvent *Event, Time time) {
+void ConvertClickToEvent(int type, int xx, int yy, int button, int state, TEvent *Event, Time mtime) {
     unsigned int myState = 0;
     static unsigned long LastClickTime = 0;
     static short LastClickCount = 0;
     static unsigned long LastClick = 0;
-    unsigned long CurTime = time;
+    unsigned long CurTime = mtime;
 
     if (type == MotionNotify) Event->What = evMouseMove;
     else if (type == ButtonPress) Event->What = evMouseDown;
@@ -901,9 +975,8 @@ void ConvertClickToEvent(int type, int xx, int yy, int button, int state, TEvent
     Event->Mouse.X = xx / FontCX;
     Event->Mouse.Y = yy / FontCY;
     if (Event->What == evMouseMove)
-        if (LastMouseX == Event->Mouse.X &&
-            LastMouseY == Event->Mouse.Y)
-        {
+	if (LastMouseX == Event->Mouse.X
+	    && LastMouseY == Event->Mouse.Y) {
             Event->What = evNone;
             return;
         }
@@ -942,9 +1015,9 @@ void ConvertClickToEvent(int type, int xx, int yy, int button, int state, TEvent
     if (state & ShiftMask) myState |= kfShift;
     if (state & ControlMask) myState |= kfCtrl;
     if (state & Mod1Mask) myState |= kfAlt;
-    if (state & Mod2Mask) myState |= kfAlt;
-    if (state & Mod3Mask) myState |= kfAlt;
-    if (state & Mod4Mask) myState |= kfAlt;
+//    if (state & Mod2Mask) myState |= kfAlt;
+//    if (state & Mod3Mask) myState |= kfAlt;
+//    if (state & Mod4Mask) myState |= kfAlt;
     Event->Mouse.KeyMask = myState;
 
     if (Event->What == evMouseDown) {
@@ -994,9 +1067,9 @@ void ProcessXEvents(TEvent *Event) {
     memset((void *)&event, 0, sizeof(event));
     Event->What = evNone;
 #ifdef WINNT
-    rc = -1;
+    //int rc = -1;
 #else
-    rc =
+    //int rc =
 #endif
         XNextEvent(display, &event);
 
@@ -1013,12 +1086,39 @@ void ProcessXEvents(TEvent *Event) {
 
     switch (event.type) {
     case Expose:
+	//fprintf(stderr, "EXPOSE\tx:%3d  y:%3d  w:%3d  h:%3d\n",
+	//	exposeEvent->x, exposeEvent->y,
+	//	exposeEvent->width, exposeEvent->height);
         UpdateWindow(exposeEvent->x,
                      exposeEvent->y,
                      exposeEvent->width,
                      exposeEvent->height);
         break;
     case GraphicsExpose:
+        /* catch up same events to speed up this a bit */
+	state = XEventsQueued(display, QueuedAfterReading);
+        // printf("Event %d\n", state);
+	while (state-- > 0) {
+	    XEvent e;
+	    XGraphicsExposeEvent *ge = (XGraphicsExposeEvent *) &e;
+
+	    if (XCheckTypedWindowEvent(display, win, GraphicsExpose, &e)) {
+		if (gexposeEvent->x == ge->x
+		    && gexposeEvent->y == ge->y
+		    && gexposeEvent->width == ge->width
+		    && gexposeEvent->height == ge->height) {
+		    // fprintf(stderr, "found the same gexpose event\n");
+		    continue;
+		} else {
+		    // fprintf(stderr, "caught different gexpose event\n");
+		    XPutBackEvent(display, &e);
+		}
+	    }
+	    break;
+	}
+	//fprintf(stderr, "GEXPOSE\tx:%3d  y:%3d  w:%3d  h:%3d\n",
+	//	gexposeEvent->x, gexposeEvent->y,
+	//	gexposeEvent->width, gexposeEvent->height);
         UpdateWindow(gexposeEvent->x,
                      gexposeEvent->y,
                      gexposeEvent->width,
@@ -1039,10 +1139,10 @@ void ProcessXEvents(TEvent *Event) {
         ConvertClickToEvent(event.type, buttonEvent->x, buttonEvent->y, buttonEvent->button, buttonEvent->state, Event, motionEvent->time);
         break;
     case FocusIn:
-        I18NFocusIn(xic);
+        i18n_focus_in(i18n_ctx);
         break;
     case FocusOut:
-        I18NFocusOut(xic);
+        i18n_focus_out(i18n_ctx);
         break;
     case KeyPress:
         // case KeyRelease:
@@ -1054,7 +1154,7 @@ void ProcessXEvents(TEvent *Event) {
         if (event.type == KeyRelease)
             XLookupString(keyEvent, keyName, sizeof(keyName), &key, 0);
         else {
-            I18NLookupString(keyEvent, keyName, sizeof(keyName), &key, xic);
+	    i18n_lookup_sym(keyEvent, keyName, sizeof(keyName), &key, i18n_ctx->xic);
             if (!key)
                 break;
         }
@@ -1103,13 +1203,14 @@ void ProcessXEvents(TEvent *Event) {
 
             if (event.xselectionrequest.selection == XA_PRIMARY &&
                 event.xselectionrequest.target == XA_STRING)
-            {
+	    {
+                static unsigned char empty[] = "";
                 XChangeProperty(display,
                                 event.xselectionrequest.requestor,
                                 event.xselectionrequest.property,
                                 event.xselectionrequest.target,
                                 8, PropModeReplace,
-                                (unsigned char *)(CurSelectionData ? CurSelectionData : ""),
+                                (CurSelectionData ? CurSelectionData : empty),
                                 CurSelectionLen);
                 notify.xselection.property = event.xselectionrequest.property;
             } else if (event.xselectionrequest.selection == XA_PRIMARY &&
@@ -1271,24 +1372,20 @@ int ConFlush(void) {
     return 0;
 }
 
-int ConGrabEvents(TEventMask EventMask) {
+int ConGrabEvents(TEventMask /*EventMask*/) {
     return 0;
 }
 
 int GetXSelection(int *len, char **data) {
     if (CurSelectionOwn) {
-        *data = (char *)malloc(CurSelectionLen);
+        *data = (char *) malloc(CurSelectionLen);
         if (*data == 0)
             return -1;
         memcpy(*data, CurSelectionData, CurSelectionLen);
         *len = CurSelectionLen;
         return 0;
     } else {
-        if (XGetSelectionOwner(display, XA_PRIMARY) == None)
-        {
-        }
-        else
-        {
+        if (XGetSelectionOwner(display, XA_PRIMARY) != None) {
             XEvent event;
             Atom type;
             long extra;
@@ -1304,19 +1401,19 @@ int GetXSelection(int *len, char **data) {
 
             time_started = time(NULL);
 
-            while (1) {
+            for (;;) {
                 if (XCheckTypedWindowEvent(display,
                                            win,
                                            SelectionNotify,
                                            &event))
                     break;
 
-                time_t now = time(NULL);
+                time_t tnow = time(NULL);
 
-                if (time_started > now)
-                    time_started = now;
+                if (time_started > tnow)
+                    time_started = tnow;
 
-                if (now - time_started > 5000)
+                if (tnow - time_started > 5000)
                     return -1;
             }
 
@@ -1327,8 +1424,8 @@ int GetXSelection(int *len, char **data) {
              event.xselection.property != None &&
              event.type != ButtonPress);*/
 
-            if (event.type == SelectionNotify && event.xselection.property != None)
-            {
+	    if (event.type == SelectionNotify
+		&& event.xselection.property != None) {
                 XGetWindowProperty(display,
                                    event.xselection.requestor,
                                    event.xselection.property,
@@ -1352,23 +1449,23 @@ int GetXSelection(int *len, char **data) {
 }
 
 int SetXSelection(int len, char *data) {
-    if (CurSelectionData != 0) {
+    if (CurSelectionData != 0)
         free(CurSelectionData);
-        CurSelectionData = 0;
-        CurSelectionLen = 0;
+
+    CurSelectionData = (unsigned char *)malloc(len);
+    if (CurSelectionData == 0) {
+	CurSelectionLen = 0;
+	return -1;
     }
-    CurSelectionData = (char *)malloc(len);
-    if (CurSelectionData == 0)
-        return -1;
     CurSelectionLen = len;
     memcpy(CurSelectionData, data, CurSelectionLen);
     if (CurSelectionLen < 64 * 1024) {
         XStoreBytes(display, data, len);
-        XSetSelectionOwner (display, XA_PRIMARY, win, CurrentTime);
+        XSetSelectionOwner(display, XA_PRIMARY, win, CurrentTime);
         if (XGetSelectionOwner(display, XA_PRIMARY) == win)
             CurSelectionOwn = 1;
     } else {
-        XSetSelectionOwner (display, XA_PRIMARY, None, CurrentTime);
+        XSetSelectionOwner(display, XA_PRIMARY, None, CurrentTime);
     }
     return 0;
 }
@@ -1405,16 +1502,26 @@ GUI::GUI(int &argc, char **argv, int XSize, int YSize) {
     argc = o;
     argv[argc] = 0;
 
-    fArgc = argc;
-    fArgv = argv;
-
-    if (::ConInit(XSize, YSize) == 0)
+    if (::ConInit(XSize, YSize) == 0
+        && SetupXWindow(argc, argv) == 0)
         gui = this;
     else
         gui = NULL;
+    fArgc = argc;
+    fArgv = argv;
 }
 
 GUI::~GUI() {
+
+    for (int i = 0; i < 256; i++)
+	XFreeGC(display, GCs[i]);
+
+    XDestroyWindow(display, win);
+    XCloseDisplay(display);
+
+    if (CurSelectionData != 0)
+        free(CurSelectionData);
+
     ::ConDone();
 }
 
@@ -1487,7 +1594,7 @@ int GUI::SetPipeView(int id, EModel *notify) {
 
 int GUI::ReadPipe(int id, void *buffer, int len) {
 #ifndef NO_PIPES
-    int rc;
+    int r;
 
     if (id < 0 || id > MAX_PIPES)
         return -1;
@@ -1495,18 +1602,18 @@ int GUI::ReadPipe(int id, void *buffer, int len) {
         return -1;
     //fprintf(stderr, "Pipe Read: Get %d %d\n", id, len);
 
-    rc = read(Pipes[id].fd, buffer, len);
+    r = read(Pipes[id].fd, buffer, len);
     //fprintf(stderr, "Pipe Read: Got %d %d\n", id, len);
-    if (rc == 0) {
+    if (r == 0) {
         close(Pipes[id].fd);
         Pipes[id].fd = -1;
         return -1;
     }
-    if (rc == -1) {
+    if (r == -1) {
         Pipes[id].stopped = 1;
         return 0;
     }
-    return rc;
+    return r;
 #else
     return 0;
 #endif
@@ -1537,22 +1644,26 @@ int GUI::ClosePipe(int id) {
 int GUI::RunProgram(int mode, char *Command) {
     char Cmd[1024];
 
+    strlcpy(Cmd, XShellCommand, sizeof(Cmd));
+
     if (*Command == 0)  // empty string = shell
-        strcpy(Cmd, "xterm &");
+        strlcat(Cmd, " -ls &", sizeof(Cmd));
     else {
-        strcpy(Cmd, "xterm -e ");
-        strcat(Cmd, Command);
+        strlcat(Cmd, " -e ", sizeof(Cmd));
+	strlcat(Cmd, Command, sizeof(Cmd));
         if (mode == RUN_ASYNC)
-            strcat(Cmd, " &");
+            strlcat(Cmd, " &", sizeof(Cmd));
     }
-    rc = system(Cmd);
-    return rc;
+    return system(Cmd);
 }
 
-char ConGetDrawChar(int index) {
-    static char tab[] = "\x0D\x0C\x0E\x0B\x12\x19____+>\x1F\x01\x12 ";
+char ConGetDrawChar(int idx) {
+    static const char *tab=NULL;
 
-    assert(index >= 0 && index < (int)strlen(tab));
+    if (!tab) {
+        tab=GetGUICharacters ("X11","\x0D\x0C\x0E\x0B\x12\x19____+>\x1F\x01\x12 ");
+    }
+    assert(idx >= 0 && idx < (int) strlen(tab));
 
-    return tab[index];
+    return tab[idx];
 }
