@@ -190,7 +190,7 @@ int ConGetEvent(TEventMask EventMask, TEvent *Event, int WaitTime, int Delete);
 int ConPutEvent(TEvent Event);
 
 void MoveCh(PCell B, char Ch, TAttr Attr, int Count);
-void MoveChar(PCell B, int Pos, int Width, char Ch, TAttr Attr, int Count);
+void MoveChar(PCell B, int Pos, int Width, const char Ch, TAttr Attr, int Count);
 void MoveMem(PCell B, int Pos, int Width, const char* Ch, TAttr Attr, int Count);
 void MoveStr(PCell B, int Pos, int Width, const char* Ch, TAttr Attr, int MaxCount);
 void MoveCStr(PCell B, int Pos, int Width, const  char* Ch, TAttr A0, TAttr A1, int MaxCount);
