@@ -8,8 +8,9 @@
  */
 
 #include "fte.h"
+#include "c_history.h"
 
-EBuffer *SS = 0; // scrap buffer (clipboard)
+EBuffer *SSBuffer = 0; // scrap buffer (clipboard)
 
 ///////////////////////////////////////////////////////////////////////////////
 
@@ -80,16 +81,22 @@ EBuffer::EBuffer(int createFlags, EModel **ARoot, const char * /*AName*/)
 
 EBuffer::~EBuffer() {
 #ifdef CONFIG_HISTORY
-    if (FileName != 0 && Loaded)
+    if (FileName != 0 && Loaded) {
         UpdateFPos(FileName, VToR(CP.Row), CP.Col);
+#ifdef CONFIG_BOOKMARKS
+        if (BFI (this,BFI_SaveBookmarks)==3) StoreBookmarks(this);
+#endif
+    }
 #endif
     if (FileName && Loaded)
         markIndex.storeForBuffer(this);
     
     Clear();
-    free(LL);
+    if (LL)
+	free(LL);
     //free(Name);
-    free(FileName);
+    if (FileName)
+	free(FileName);
 #ifdef CONFIG_BOOKMARKS
     if (BMCount != 0) {
         for (int i = 0; i < BMCount; i++)
@@ -104,6 +111,7 @@ EBuffer::~EBuffer() {
         free(rlst.Lines);
         rlst.Lines = 0;
     }
+    DeleteRelated();
 #endif
 }
 
@@ -121,6 +129,15 @@ int EBuffer::Clear() {
 #ifdef CONFIG_SYNTAX_HILIT
     EndHilit = -1;
     StartHilit = 0;
+
+    while (WordCount--)
+    {
+        free(WordList[WordCount]);
+    }
+    free(WordList);
+
+    WordCount = 0;
+    WordList = 0;
 #endif
 #ifdef CONFIG_OBJ_ROUTINE
     rlst.Count = 0;
@@ -129,16 +146,20 @@ int EBuffer::Clear() {
         rlst.Lines = 0;
     }
 #endif
-    for (int I = 0; I < RCount; I++)
-        delete LL[GapLine(I, RGap, RCount, RAllocated)];
+    if (LL)
+    {
+	for (int i = 0; i < RCount; i++)
+	    delete LL[GapLine(i, RGap, RCount, RAllocated)];
+	free(LL);
+	LL = 0;
+    }
     RCount = RAllocated = RGap = 0;
-    if (LL) 
-        free(LL);
-    LL = 0;
     VCount = VAllocated = VGap = 0;
     if (VV)
+    {
         free(VV);
-    VV = 0;
+	VV = 0;
+    }
 #ifdef CONFIG_UNDOREDO
     FreeUndo();
 #endif
@@ -167,6 +188,26 @@ int EBuffer::FreeUndo() {
 #endif
 
 int EBuffer::Modify() {
+    // if RecheckReadOnly is activated do readonly checking when necessary
+    if (RecheckReadOnly != 0)
+    {
+        if (BFI(this, BFI_ReadOnly)) {
+            // File might have been toggled writable outside the editor, or
+            //  you might do what I do, and do a Tools/Run/"p4 edit Filename.cpp"
+            //  from inside FTE, and it's a pain to manually reopen the file, so
+            //  recheck writability here instead. Note that we don't check the
+            //  converse, since in reality this is rarely a problem, and the
+            //  file save routines will check this (oh well).  --ryan.
+            struct stat StatBuf;
+            if ((FileName != 0) && FileOk && (stat(FileName, &StatBuf) == 0)) {
+                if (!(StatBuf.st_mode & (S_IWRITE | S_IWGRP | S_IWOTH)))
+                    BFI(this, BFI_ReadOnly) = 1;
+                else
+                    BFI(this, BFI_ReadOnly) = 0;
+            }
+        }
+    }
+
     if (BFI(this, BFI_ReadOnly)) {
         Msg(S_ERROR, "File is read-only.");
         return 0;
@@ -211,7 +252,7 @@ int EBuffer::LoadRegion(EPoint * /*A*/, int /*FH*/, int /*StripChar*/, int /*Lin
     return 0;
 }
 
-int EBuffer::InsertLine(EPoint Pos, int ACount, char *AChars) {
+int EBuffer::InsertLine(EPoint Pos, int ACount, const char *AChars) {
     if (InsLine(Pos.Row, 0) == 0) return 0;
     if (InsText(Pos.Row, Pos.Col, ACount, AChars) == 0) return 0;
     return 1;
@@ -576,6 +617,7 @@ int EBuffer::DelLine(int Row, int DoMark) {
         if (MoveRGap(Row) == 0) return 0;
     
     GapSize = RAllocated - RCount;
+
     delete LL[RGap + GapSize];
     LL[RGap + GapSize] = 0;
     RCount--;
@@ -609,6 +651,8 @@ int EBuffer::InsLine(int Row, int DoAppend, int DoMark) {
     
     //    printf("InsLine: %d\n", Row);
     
+    //if (! LL)
+    //   return 0;
     if (Row < 0) return 0;
     if (Row > RCount) return 0;
     if (Modify() == 0) return 0;
@@ -708,7 +752,7 @@ int EBuffer::DelChars(int Row, int Ofs, int ACount) {
     return 1;
 }
 
-int EBuffer::InsChars(int Row, int Ofs, int ACount, char *Buffer) {
+int EBuffer::InsChars(int Row, int Ofs, int ACount, const char *Buffer) {
     PELine L;
     
 //    printf("InsChars: %d:%d %d\n", Row, Ofs, ACount);
@@ -766,7 +810,7 @@ int EBuffer::UnTabPoint(int Row, int Col) {
     return 1;
 }
 
-int EBuffer::ChgChars(int Row, int Ofs, int ACount, char * /*Buffer*/) {
+int EBuffer::ChgChars(int Row, int Ofs, int ACount, const char * /*Buffer*/) {
     PELine L;
     
     assert(Row >= 0 && Row < RCount && Ofs >= 0);
@@ -824,7 +868,7 @@ int EBuffer::DelText(int Row, int Col, int ACount, int DoMark) {
     return 1;
 }
 
-int EBuffer::InsText(int Row, int Col, int ACount, char *ABuffer, int DoMark) {
+int EBuffer::InsText(int Row, int Col, int ACount, const char *ABuffer, int DoMark) {
     int B, L;
     
 //    printf("InsText: %d:%d %d\n", Row, Col, ACount);
