@@ -834,7 +834,7 @@ int EBuffer::TypeChar(char aCh) { // does abbrev expansion if appropriate
             Len = P1 - P;
             C = ScreenPos(L, P);
             assert(C1 - C == Len);
-            if (Len > 0 && Len < int(sizeof(Str))) {
+	    if (Len > 0 && Len < int (sizeof(Str))) {
                 //fprintf(stderr, "TypeChar 2\n");
                 memcpy(Str, L->Chars + P, Len);
                 Str[Len] = 0;
@@ -868,7 +868,7 @@ int EBuffer::TypeChar(char aCh) { // does abbrev expansion if appropriate
     return InsertString(&aCh, 1);
 }
 
-int EBuffer::InsertString(char *aStr, int aCount) {
+int EBuffer::InsertString(const char *aStr, int aCount) {
     int P;
     int C, L;
     int Y = VToR(CP.Row);
@@ -947,33 +947,38 @@ int EBuffer::InsertSpacesToTab(int TSize) {
 }
 
 int EBuffer::InsertTab() {
-    if (BFI(this, BFI_SpaceTabs)) {
-        return InsertSpacesToTab(BFI(this, BFI_TabSize));
-    } else
-        return InsertChar(9);
+    return (BFI(this, BFI_SpaceTabs)) ?
+	InsertSpacesToTab(BFI(this, BFI_TabSize)) : InsertChar(9);
 }
 
 int EBuffer::InsertSpace() {
     return TypeChar(32);
 }
 
-int EBuffer::LineIndented(int Row) {
-    int P;
-    char *PC;
-    int I;
-    int Ind = 0;
+int EBuffer::LineIndented(int Row, const char *indentchars) {
+    ELine *l;
     
     if (Row < 0) return 0;
     if (Row >= RCount) return 0;
-    P = RLine(Row)->Count;
-    PC = RLine(Row)->Chars;
-    
-    for(I = 0; I < P; I++) {
-        if (PC[I] == ' ') Ind++;
-        else if ((PC[I] == 9) && (BFI(this, BFI_ExpandTabs) == 1)) Ind = NextTab(Ind, BFI(this, BFI_TabSize));
-        else break;
+    l = RLine(Row);
+    return ScreenPos(l, LineIndentedCharCount(l, indentchars));
+}
+
+int EBuffer::LineIndentedCharCount(ELine *l, const char *indentchars) {
+    char *PC;
+    int CC, i;
+
+    if (! l)
+        return 0;
+    if (! indentchars)
+        indentchars = " \t";
+    CC = l->Count;
+    PC = l->Chars;
+    for(i = 0; i < CC; i++) {
+        if (! strchr(indentchars, PC[i]))
+            break;
     }
-    return Ind;
+    return i;
 }
 
 int EBuffer::IndentLine(int Row, int Indent) {
@@ -1030,7 +1035,7 @@ int EBuffer::IsLineBlank(int Row) {
 }
 
 #ifdef CONFIG_WORDWRAP
-#define WFAIL(x) return 0 /*do { puts(#x "\x7"); return -1; } while (0)*/
+#define WFAIL(x) return 0	/*do { puts(#x "\x7"); return -1; } while (0) */
 
 int EBuffer::DoWrap(int WrapAll) {
     int L, Len, C, P, Ind;
@@ -1282,13 +1287,15 @@ int EBuffer::ToggleIndentWithTabs() { TOGGLE(IndentWithTabs); }
 int EBuffer::ToggleBackSpUnindents() { TOGGLE(BackSpUnindents); }
 int EBuffer::ToggleTrim() { TOGGLE(Trim); }
 int EBuffer::ToggleShowMarkers() { TOGGLE_R(ShowMarkers); }
+int EBuffer::ToggleHilitTags() { TOGGLE_R(HilitTags); }
+int EBuffer::ToggleShowBookmarks() { TOGGLE_R(ShowBookmarks); }
 
 int EBuffer::ToggleWordWrap() { 
     BFI(this, BFI_WordWrap) = (BFI(this, BFI_WordWrap) + 1) % 3;
     /*Msg(INFO,
         "WordWrap is now %s.",
         (BFI(this, BFI_WordWrap) == 2) ? "AUTO" :
-        (BFI(this, BFI_WordWrap) == 1) ? "ON" : "OFF");*/
+       (BFI(this, BFI_WordWrap) == 1) ? "ON" : "OFF"); */
     return 1;
 }
 
