@@ -44,10 +44,19 @@ int EBuffer::GetMap(int Row, int *StateLen, hsState **StateMap) {
 void EBuffer::FullRedraw() { // redraw all views
     EView *V = View;
     EEditPort *W;
+    int Min, Max;
 
     while (V) {
         W = GetViewVPort(V);
-        Draw(W->TP.Row, W->TP.Row + W->Rows);
+        // Need to use real lines, not virtual
+        // (similar to HilitMatchBracket)
+        Min = VToR(W->TP.Row);
+        Max = W->TP.Row + W->Rows;
+        if (Max >= VCount)
+            Max = RCount;
+        else
+            Max = VToR(Max);
+        Draw(Min, Max);
         V = V->Next;
         if (V == View)
             break;
@@ -122,7 +131,8 @@ void EBuffer::Draw(int Row0, int RowE) {
     if (RowE == -1) {
         RedrawToEos = 1;
         MaxRedraw = MinRedraw;
-    } else if ((RowE > MaxRedraw) || (MaxRedraw == -1)) if (RowE != -1) MaxRedraw = RowE;
+    } else if (((RowE > MaxRedraw) || (MaxRedraw == -1)) && (RowE != -1))
+	MaxRedraw = RowE;
     //    printf("m = %d, max = %d, rts = %d\n", MinRedraw, MaxRedraw, RedrawToEos);
 }
 
@@ -165,9 +175,12 @@ void EBuffer::DrawLine(TDrawBuffer B, int VRow, int C, int W, int &HilitX) {
 
             f = FindFold(Row);
             if (f != -1) {
+                int foldColor;
+                if (FF[f].level<5) foldColor=hcPlain_Folds[FF[f].level];
+                else foldColor=hcPlain_Folds[4];
                 if (FF[f].open == 1) {
                     l = sprintf(fold, "[%d]", FF[f].level);
-                    MoveStr(B, ECol - C + 1, W, fold, hcPlain_Folds, 10);
+                    MoveStr(B, ECol - C + 1, W, fold, foldColor, 10);
                     ECol += l;
                 } else {
                     if (VRow < VCount - 1) {
@@ -176,9 +189,9 @@ void EBuffer::DrawLine(TDrawBuffer B, int VRow, int C, int W, int &HilitX) {
                         Folded = RCount - (VRow + Vis(VRow));
                     }
                     l = sprintf(fold, "(%d:%d)", FF[f].level, Folded);
-                    MoveStr(B, ECol - C + 1, W, fold, hcPlain_Folds, 10);
+                    MoveStr(B, ECol - C + 1, W, fold, foldColor, 10);
                     ECol += l;
-                    MoveAttr(B, 0, W, hcPlain_Folds, W);
+                    MoveAttr(B, 0, W, foldColor, W);
                 }
             }
         }
@@ -218,6 +231,23 @@ void EBuffer::DrawLine(TDrawBuffer B, int VRow, int C, int W, int &HilitX) {
             else
                 MoveAttr(B, StartPos, W, hcPlain_Selected, EndPos - StartPos);
         }
+#ifdef CONFIG_BOOKMARKS
+        if (BFI(this, BFI_ShowBookmarks)) {
+            int i = 0;
+            char *Name;
+            EPoint P;
+            while ((i = GetBookmarkForLine(i, Row, Name, P)) != -1) {
+                if (strncmp(Name, "_BMK", 4) == 0) {
+                    // User bookmark, hilite line
+                    if (BFI(this, BFI_SeeThruSel))
+                        MoveBgAttr(B, 0, W, hcPlain_Bookmark, W);
+                    else
+                        MoveAttr(B, 0, W, hcPlain_Bookmark, W);
+                    break;
+                }
+            }
+        }
+#endif
         if (Match.Row != -1 && Match.Col != -1) {
             if (Row == Match.Row) {
                 if (BFI(this, BFI_SeeThruSel))
@@ -314,7 +344,7 @@ void EBuffer::Redraw() {
                         if (DeltaY < 0) {
                             W->ScrollY(DeltaY);
                             A = W->TP.Row;
-                            B = W->TP.Row - DeltaY;
+			    B = W->TP.Row - DeltaY;
                         } else {
                             W->ScrollY(DeltaY);
                             A = W->TP.Row + Rows - DeltaY;
@@ -334,7 +364,7 @@ void EBuffer::Redraw() {
                     }
                     if (A < 0) A = 0;
                     if (B < 0) B = 0;
-                    Draw(VToR(A) + Delta1, VToR(B) + Delta2);
+		    Draw(VToR(A) + Delta1, VToR(B) + Delta2);
                 }
             } else {
                 int A = W->TP.Row;
@@ -422,6 +452,7 @@ void EBuffer::Redraw() {
                          (BlockMode == bmLine) ? 'L': 'C'
                         ),
 #ifdef CONFIG_WORDWRAP
+                        (BFI(this, BFI_WordWrap) == 3) ? 't' :
                         (BFI(this, BFI_WordWrap) == 2) ? 'W' :
                         (BFI(this, BFI_WordWrap) == 1) ? 'w' :
                         ' ',
@@ -546,7 +577,7 @@ int EBuffer::GetHilitWord(int len, char *str, ChColor &clr, int IgnCase) {
         memcpy(s, str, len);
         s[len] = 0;
         if (HilitFindWord(s)) {
-            clr = hcPlain_HilitWord;
+	    clr = COUNT_CLR + hcPlain_HilitWord;
             return 1;
         }
     }
@@ -556,7 +587,7 @@ int EBuffer::GetHilitWord(int len, char *str, ChColor &clr, int IgnCase) {
     if (IgnCase) {
         while (p && *p) {
             if (strnicmp(p, str, len) == 0) {
-                clr = p[len];
+		clr = COUNT_CLR + ((unsigned char*)p)[len];
                 return 1;
             }
             p += len + 1;
@@ -564,8 +595,9 @@ int EBuffer::GetHilitWord(int len, char *str, ChColor &clr, int IgnCase) {
     } else {
         while (p && *p) {
             if (memcmp(p, str, len) == 0) {
-                clr = p[len];
-                return 1;
+                clr = COUNT_CLR + ((unsigned char*)p)[len];
+                //printf("PLEN %d  %d\n", p[len], COUNT_CLR);
+		return 1;
             }
             p += len + 1;
         }
@@ -575,8 +607,9 @@ int EBuffer::GetHilitWord(int len, char *str, ChColor &clr, int IgnCase) {
 
         memcpy(s, str, len);
         s[len] = 0;
-        if (TagDefined(s)) {
-            clr = 0x0A;
+        if (BFI(this, BFI_HilitTags)&&TagDefined(s)) {
+	    //clr = 0x0A;
+	    clr = CLR_HexNumber;// Mode->fColorize->Colors[];
             return 1;
         }
     }
