@@ -27,7 +27,6 @@ public:
     EKeyMap *ActiveMap;
     EKeyMap *OverrideMap;
     char CharMap[32];
-    ExState State;
 
     EGUI(int &argc, char **argv, int XSize, int YSize);
     virtual ~EGUI();
@@ -66,6 +65,7 @@ public:
     
     int ShowEntryScreen();
     int RunProgram(ExState &State, GxView *view);
+    int RunProgramAsync(ExState &State, GxView *view);
     
     int MainMenu(ExState &State, GxView *View);
     int ShowMenu(ExState &State, GxView *View);
