@@ -13,12 +13,23 @@
 #include <assert.h>
 #include <signal.h>
 #include <stdarg.h>
+#if defined(MSVC)
+#include <malloc.h>
+#endif
 
 #include "console.h"
 #include "gui.h"
 #include "c_mode.h"
 #include "c_color.h"
 
+#if defined(_DEBUG) && defined(MSVC) && defined(MSVCDEBUG)
+#include <crtdbg.h>
+
+#define new new( _CLIENT_BLOCK, __FILE__, __LINE__)
+
+#endif //_DEBUG && MSVC && MSVCDEBUG
+
+
 int ShowVScroll = 1;
 int ShowHScroll = 1;
 int ShowMenuBar = 1;
@@ -587,7 +598,8 @@ GFrame::~GFrame() {
         frames = Next;
     }
     Next = Prev = 0;
-    free(Menu);
+    if (Menu)
+	free(Menu);
 }
 
 int GFrame::ConSetTitle(char *Title, char *STitle) {
@@ -702,12 +714,12 @@ void GFrame::Repaint() {
     }
 }
 
-void GFrame::InsertView(GView *Prev, GView *view) {
+void GFrame::InsertView(GView *prev, GView *view) {
     if (!view) return ;
-    if (Prev) {
-        view->Prev = Prev;
-        view->Next = Prev->Next;
-        Prev->Next = view;
+    if (prev) {
+        view->Prev = prev;
+        view->Next = prev->Next;
+	prev->Next = view;
         view->Next->Prev = view;
     } else {
         view->Prev = view->Next = view;
