@@ -7,6 +7,7 @@
  *
  */
 
+#define NEED_LOG_H
 #include "fte.h"
 
 #ifdef CONFIG_HILIT_C
@@ -14,6 +15,8 @@
 
 #define PRINTF(x) //printf x
 
+#define ISNAME(x)  (isalnum(x) || (x == '_'))
+
 
 #define hsC_Normal       0
 #define hsC_Comment      1
@@ -28,10 +31,9 @@
 #define hsC_CPP_ABrace  16
 
 int Hilit_C(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line, hlState &State, hsState *StateMap, int *ECol) {
-    ChColor *Colors = BF->Mode->fColorize->Colors;
     int j = 0;
     int firstnw = 0;
-    HILIT_VARS(Colors[CLR_Normal], Line);
+    HILIT_VARS(BF->Mode->fColorize->Colors, Line);
     int len1 = len;
     char *last = p + len1 - 1;
     int was_include = 0;
@@ -42,33 +44,44 @@ int Hilit_C(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line, h
             switch(State) {
             default:
             case hsC_Normal:
-                if (isalpha(*p) || *p == '_') {
+                if (toupper(*p) == 'L' && p[1] == '"') {
+                    State = hsC_String2;
+                    Color = CLR_String;
+                    goto hilit2;
+                } else if (toupper(*p) == 'L' && p[1] == '\'') {
+                    State = hsC_String1;
+                    Color = CLR_String;
+                    goto hilit2;
+                } else if (isalpha(*p) || *p == '_') {
                     j = 0;
-                    while (((i + j) < Line->Count) &&
-                           (isalnum(Line->Chars[i+j]) ||
-                            (Line->Chars[i + j] == '_'))
-                          ) j++;
+		    while (((i + j) < Line->Count)
+			   && ISNAME(Line->Chars[i + j]))
+			j++;
                     if (BF->GetHilitWord(j, &Line->Chars[i], Color)) {
-                        //                        Color = hcC_Keyword;
-                        State = hsC_Keyword;
+			//Color = hcC_Keyword;
+			State = hsC_Keyword;
                     } else {
                         int x;
                         x = i + j;
-                        while ((x < Line->Count) &&
-                               ((Line->Chars[x] == ' ') || (Line->Chars[x] == 9))) x++;
-                        if ((x < Line->Count) && (Line->Chars[x] == '(')) {
-                            Color = Colors[CLR_Function];
-                        } else if ((x < Line->Count) && (Line->Chars[x] == ':' && (x == Line->Count - 1 || Line->Chars[x + 1] != ':')) && firstnw == 1) {
-                            Color = Colors[CLR_Label];
+			while (x < Line->Count && isspace(Line->Chars[x]))
+			    x++;
+			if (x < Line->Count && Line->Chars[x] == '(') {
+                            Color = CLR_Function;
+			} else if ((x < Line->Count)
+				   && (Line->Chars[x] == ':'
+				       && (x == Line->Count - 1
+                                           || Line->Chars[x + 1] != ':'))
+                                   && firstnw == 1) {
+                            Color = CLR_Label;
                         } else {
-                            Color = Colors[CLR_Normal];
+                            Color = CLR_Normal;
                         }
                         State = hsC_Normal;
                     }
                     if (StateMap)
                         memset(StateMap + i, State, j);
                     if (B)
-                        MoveMem(B, C - Pos, Width, Line->Chars + i, Color, j);
+                        MoveMem(B, C - Pos, Width, Line->Chars + i, HILIT_CLRD(), j);
                     i += j;
                     len -= j;
                     p += j;
@@ -77,47 +90,74 @@ int Hilit_C(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line, h
                     continue;
                 } else if ((len >= 2) && (*p == '/') && (*(p+1) == '*')) {
                     State = hsC_Comment;
-                    Color = Colors[CLR_Comment];
+                    Color = CLR_Comment;
                     goto hilit2;
-                } else if ((len >= 2) && (*p == '/') && (*(p+1) == '/')) {
+                } else if ((len >= 2) && (*p == '/') && (p[1] == '/')) {
                     State = hsC_CommentL;
-                    Color = Colors[CLR_Comment];
+                    Color = CLR_Comment;
                     goto hilit2;
-                } else if ((len >= 2) && (*p == '0') && (*(p+1) == 'x')) {
-                    Color = Colors[CLR_HexNumber];
-                    ColorNext();
-                    ColorNext();
-                    while (len && isxdigit(*p)) ColorNext();
-                    if (len && (toupper(*p) == 'U')) ColorNext();
-                    if (len && (toupper(*p) == 'L')) ColorNext();
-                    continue;
-                } else if (isdigit(*p)) {
-                    Color = Colors[CLR_Number];
-                    ColorNext();
-                    while (len && (isdigit(*p) || *p == 'e' || *p == 'E' || *p == '.')) ColorNext();
-                    if (len && (toupper(*p) == 'U')) ColorNext();
-                    if (len && (toupper(*p) == 'L')) ColorNext();
+		} else if (isdigit(*p)) {
+                    // check if it is not floating point number 0.08!
+                    if ((len >= 2) && (*p == '0') && p[1] != '.') {
+                        if (toupper(*(p+1)) == 'X') {
+                            Color = CLR_HexNumber;
+                            ColorNext();
+                            ColorNext();
+                            while (len && isxdigit(*p)) ColorNext();
+                        } else /* assume it's octal */ {
+                            Color = CLR_Number;
+                            ColorNext();
+                            while (len && ('0' <= *p && *p <= '7')) ColorNext();
+                            // if we hit a non-octal, stop hilighting it.
+                            if (len && ('8' <= *p && *p <= '9'))
+                            {
+                                Color = CLR_Normal;
+                                while (len && !isspace(*p)) ColorNext();
+                                continue;
+                            }
+                        }
+                    } else /* assume it's decimal/floating */ {
+                        Color = CLR_Number;
+                        ColorNext();
+                        while (len && (isdigit(*p) || *p == 'e' || *p == 'E' || *p == '.')) ColorNext();
+                        // if it ends with 'f', the number can't have more extras.
+                        if (len && (toupper(*p) == 'F')) {
+                            ColorNext();
+                            continue;
+                        }
+                    }
+                    // allowed extras: u, l, ll, ul, ull, lu, llu
+                    int colored_u = 0;
+                    if (len && (toupper(*p) == 'U')) {
+                        ColorNext();
+                        colored_u = 1;
+                    }
+                    if (len && (toupper(*p) == 'L')) {
+                        ColorNext();
+                        if (len && (toupper(*p) == 'L')) ColorNext();
+                        if (! colored_u && len && (toupper(*p) == 'U')) ColorNext();
+                    }
                     continue;
                 } else if (*p == '\'') {
                     State = hsC_String1;
-                    Color = Colors[CLR_String];
+                    Color = CLR_String;
                     goto hilit;
                 } else if (*p == '"') {
                     State = hsC_String2;
-                    Color = Colors[CLR_String];
+                    Color = CLR_String;
                     goto hilit;
                 } else if (firstnw == 1 && *p == '#') {
                     State = hsC_CPP;
-                    Color = Colors[CLR_CPreprocessor];
+                    Color = CLR_CPreprocessor;
                     goto hilit;
                 } else if (ispunct(*p) && *p != '_') {
-                    Color = Colors[CLR_Punctuation];
+                    Color = CLR_Punctuation;
                     goto hilit;
                 }
-                Color = Colors[CLR_Normal];
+                Color = CLR_Normal;
                 goto hilit;
             case hsC_Comment:
-                Color = Colors[CLR_Comment];
+                Color = CLR_Comment;
                 if ((len >= 2) && (*p == '*') && (*(p+1) == '/')) {
                     ColorNext();
                     ColorNext();
@@ -126,7 +166,7 @@ int Hilit_C(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line, h
                 }
                 goto hilit;
             case hsC_CPP_Comm:
-                Color = Colors[CLR_Comment];
+                Color = CLR_Comment;
                 if ((len >= 2) && (*p == '*') && (*(p+1) == '/')) {
                     ColorNext();
                     ColorNext();
@@ -135,17 +175,17 @@ int Hilit_C(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line, h
                 }
                 goto hilit;
             case hsC_CPP_ABrace:
-                Color = Colors[CLR_String];
+                Color = CLR_String;
                 if (*p == '>') {
-                    Color = Colors[CLR_CPreprocessor];
+                    Color = CLR_CPreprocessor;
                     State = hsC_CPP;
                 }
                 goto hilit;
             case hsC_CommentL:
-                Color = Colors[CLR_Comment];
+                Color = CLR_Comment;
                 goto hilit;
             case hsC_String1:
-                Color = Colors[CLR_String];
+                Color = CLR_String;
                 if ((len >= 2) && (*p == '\\')) {
                     goto hilit2;
                 } else if (*p == '\'') {
@@ -155,7 +195,7 @@ int Hilit_C(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line, h
                 }
                 goto hilit;
             case hsC_String2:
-                Color = Colors[CLR_String];
+                Color = CLR_String;
                 if ((len >= 2) && (*p == '\\')) {
                     goto hilit2;
                 } else if (*p == '"') {
@@ -165,7 +205,7 @@ int Hilit_C(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line, h
                 }
                 goto hilit;
             case hsC_CPP_String1:
-                Color = Colors[CLR_String];
+                Color = CLR_String;
                 if ((len >= 2) && (*p == '\\')) {
                     goto hilit2;
                 } else if (*p == '\'') {
@@ -175,7 +215,7 @@ int Hilit_C(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line, h
                 }
                 goto hilit;
             case hsC_CPP_String2:
-                Color = Colors[CLR_String];
+                Color = CLR_String;
                 if ((len >= 2) && (*p == '\\')) {
                     goto hilit2;
                 } else if (*p == '"') {
@@ -185,9 +225,9 @@ int Hilit_C(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line, h
                 }
                 goto hilit;
             case hsC_CPP:
-                if (isalpha(*p) || *p == '_') {
+                if (ISNAME(*p)) {
                     j = 0;
-                    Color = Colors[CLR_CPreprocessor];
+                    Color = CLR_CPreprocessor;
                     while (((i + j) < Line->Count) &&
                            (isalnum(Line->Chars[i+j]) ||
                             (Line->Chars[i + j] == '_'))
@@ -197,7 +237,7 @@ int Hilit_C(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line, h
                     if (StateMap)
                         memset(StateMap + i, State, j);
                     if (B)
-                        MoveMem(B, C - Pos, Width, Line->Chars + i, Color, j);
+                        MoveMem(B, C - Pos, Width, Line->Chars + i, HILIT_CLRD(), j);
                     i += j;
                     len -= j;
                     p += j;
@@ -205,45 +245,71 @@ int Hilit_C(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line, h
                     continue;
                 } else if ((len >= 2) && (*p == '/') && (*(p+1) == '*')) {
                     State = hsC_CPP_Comm;
-                    Color = Colors[CLR_Comment];
+                    Color = CLR_Comment;
                     goto hilit2;
                 } else if ((len >= 2) && (*p == '/') && (*(p+1) == '/')) {
                     State = hsC_CommentL;
-                    Color = Colors[CLR_Comment];
+                    Color = CLR_Comment;
                 hilit2:
                     ColorNext();
                 hilit:
                     ColorNext();
                     continue;
-                } else if ((len >= 2) && (*p == '0') && (*(p+1) == 'x')) {
-                    Color = Colors[CLR_HexNumber];
-                    ColorNext();
-                    ColorNext();
-                    while (len && isxdigit(*p)) ColorNext();
-                    if (len && (toupper(*p) == 'U')) ColorNext();
-                    if (len && (toupper(*p) == 'L')) ColorNext();
-                    continue;
                 } else if (isdigit(*p)) {
-                    Color = Colors[CLR_Number];
-                    ColorNext();
-                    while (len && isdigit(*p)) ColorNext();
-                    if (len && (toupper(*p) == 'U')) ColorNext();
-                    if (len && (toupper(*p) == 'L')) ColorNext();
+                    if ((len >= 2) && (*p == '0')) {
+                        if (toupper(*(p+1)) == 'X') {
+                            Color = CLR_HexNumber;
+                            ColorNext();
+                            ColorNext();
+                            while (len && isxdigit(*p)) ColorNext();
+                        } else /* assume it's octal */ {
+                            Color = CLR_Number;
+                            ColorNext();
+                            while (len && ('0' <= *p && *p <= '7')) ColorNext();
+                            // if we hit a non-octal, stop hilighting it.
+                            if (len && ('8' <= *p && *p <= '9'))
+                            {
+                                Color = CLR_Normal;
+                                while (len && !isspace(*p)) ColorNext();
+                                continue;
+                            }
+                        }
+                    } else /* assume it's decimal/floating */ {
+                        Color = CLR_Number;
+                        ColorNext();
+                        while (len && (isdigit(*p) || *p == 'e' || *p == 'E' || *p == '.')) ColorNext();
+                        // if it ends with 'f', the number can't have more extras.
+                        if (len && (toupper(*p) == 'F')) {
+                            ColorNext();
+                            continue;
+                        }
+                    }
+                    // allowed extras: u, l, ll, ul, ull, lu, llu
+                    int colored_u = 0;
+                    if (len && (toupper(*p) == 'U')) {
+                        ColorNext();
+                        colored_u = 1;
+                    }
+                    if (len && (toupper(*p) == 'L')) {
+                        ColorNext();
+                        if (len && (toupper(*p) == 'L')) ColorNext();
+                        if (! colored_u && len && (toupper(*p) == 'U')) ColorNext();
+                    }
                     continue;
                 } else if (*p == '\'') {
                     State = hsC_CPP_String1;
-                    Color = Colors[CLR_String];
+                    Color = CLR_String;
                     goto hilit;
                 } else if (*p == '"') {
                     State = hsC_CPP_String2;
-                    Color = Colors[CLR_String];
+                    Color = CLR_String;
                     goto hilit;
                 } else if (*p == '<' && was_include) {
                     ColorNext();
                     State = hsC_CPP_ABrace;
                     continue;
                 } else {
-                    Color = Colors[CLR_CPreprocessor];
+                    Color = CLR_CPreprocessor;
                     goto hilit;
                 }
             }
@@ -274,24 +340,35 @@ int IsState(hsState *Buf, hsState State, int Len) {
     return 1;
 }
 
-int LookAt(EBuffer *B, int Row, unsigned int Pos, const char *What, hsState State, int NoWord) {
-    char *P;
-    unsigned int L;
-    //int StateLen;
-    //hsState *StateMap;
+int LookAt(EBuffer *B, int Row, unsigned int Pos, const char *What, hsState State, int NoWord, int CaseInsensitive) {
+    STARTFUNC("LookAt{h_c.cpp}");
+
     int Len = strlen(What);
 
-    if (Row < 0 || Row >= B->RCount) return 0;
-    P = B->RLine(Row)->Chars;
-    L = B->RLine(Row)->Count;
+    if (Row < 0 || Row >= B->RCount) {
+        LOG << "Row out of range: " << Row << " vs " << B->RCount << ENDLINE;
+        ENDFUNCRC(0);
+    }
+    char*        pLine       = B->RLine(Row)->Chars;
+    unsigned int uLineLength = B->RLine(Row)->Count;
     Pos = B->CharOffset(B->RLine(Row), Pos);
-    if (Pos + strlen(What) > L) return 0;
-    if (NoWord && L > Pos + Len && isalnum(P[Pos + Len]))
-        return 0;
-    if (memcmp(P + Pos, What, Len) == 0)
-        return 1;
+    if (Pos + strlen(What) > uLineLength) { ENDFUNCRC(0); }
+    if (NoWord && uLineLength > Pos + Len && ISNAME(pLine[Pos + Len]))
+    {
+        ENDFUNCRC(0);
+    }
+    LOG << "Check against [" << What << ']' << ENDLINE;
+    if (
+        (CaseInsensitive && memicmp(pLine + Pos, What, Len) == 0) ||
+        (!CaseInsensitive && memcmp(pLine + Pos, What, Len) == 0)
+       )
+    {
+        ENDFUNCRC(1);
+    }
     else
-        return 0;
+    {
+        ENDFUNCRC(0);
+    }
 }
 
 #ifdef CONFIG_INDENT_C
@@ -309,6 +386,7 @@ int C_CommentDelta = 1;
 int C_FirstLevelWidth = -1;
 int C_FirstLevelIndent = 4;
 int C_Continuation = 4;
+int FunctionUsesContinuation = 0;
 
 // this is global, unfortunately -- FIX !!!
 int EBuffer::SetCIndentStyle(ExState &State) {
@@ -379,6 +457,7 @@ static int CheckLabel(EBuffer *B, int Line) {
 
     while ((P < L->Count) &&
            (L->Chars[P] == ' ' || L->Chars[P] == 9)) P++;
+
     while (P < L->Count) {
         if (Cnt > 0)
             if (L->Chars[P] == ':' && (Cnt == 1 || L->Chars[P + 1] != ':')) return 1;
@@ -431,21 +510,21 @@ static int SearchBackMatch(int Count, EBuffer *B, int Row, hsState State, const
                                     didMatch = 1;
                                 else {
                                     *OPos = B->ScreenPos(B->RLine(Row), Pos);
-                                    *OLine = Row;
-                                    free(StateMap);
-                                    return B->LineIndented(Row);
-                                }
-                            }
-                        }
-                        if (LClose + Pos <= L) {
-                            if (IsState(StateMap + Pos, State, LClose)) {
-                                if (memcmp(P + Pos, Close, LClose) == 0) Count--;
-                            }
-                        }
-                    }
-                }
-            }
-            Pos--;
+				    *OLine = Row;
+				    free(StateMap);
+				    return B->LineIndented(Row);
+				}
+			    }
+			}
+			if (LClose + Pos <= L) {
+			    if (IsState(StateMap + Pos, State, LClose)) {
+				if (memcmp(P + Pos, Close, LClose) == 0) Count--;
+			    }
+			}
+		    }
+		}
+	    }
+	    Pos--;
         }
         if (bolOnly && didMatch && CountX[1] == 0 && CountX[2] == 0) {
             *OPos = 0;
@@ -459,12 +538,14 @@ static int SearchBackMatch(int Count, EBuffer *B, int Row, hsState State, const
     return -1;
 }
 
-static int FindPrevIndent(EBuffer *B, int &RowP, int &ColP, char &CharP, int Flags) {
+static int FindPrevIndent(EBuffer *B, int &RowP, int &ColP, char &CharP, int Flags)
+{
+    STARTFUNC("FindPrevIndent{h_c.cpp}");
+    LOG << "Flags: " << hex << Flags << dec << ENDLINE;
     int StateLen;
     hsState *StateMap = 0;
     char *P;
     int L;
-
     int Count[4] = {
         0, // { }
         0, // ( )
@@ -488,10 +569,18 @@ static int FindPrevIndent(EBuffer *B, int &RowP, int &ColP, char &CharP, int Fla
         L = B->RLine(RowP)->Count;
         StateMap = NULL;
         if (B->GetMap(RowP, &StateLen, &StateMap) == 0)
-            return 0;
-        if (L > 0) while (ColP >= 0) {
-
+        {
+            LOG << "Can't get state maps" << ENDLINE;
+            ENDFUNCRC(0);
+        }
+	if (L > 0)
+	    while (ColP >= 0) {
+            LOG << "ColP: " << ColP << " State: " << (int)StateMap[ColP] << ENDLINE;
             if (StateMap[ColP] == hsC_Normal) {
+                LOG << "CharP: " << BinChar(P[ColP]) << " BolChar: " << BinChar(BolChar) <<
+                    " BolRow: " << BolRow <<
+                    " BolCol: " << BolCol <<
+                    ENDLINE;
                 switch (CharP = P[ColP]) {
                 case '{':
                     if (BolChar == ':' || BolChar == ',') {
@@ -499,12 +588,11 @@ static int FindPrevIndent(EBuffer *B, int &RowP, int &ColP, char &CharP, int Fla
                         ColP = BolCol;
                         RowP = BolRow;
                         free(StateMap);
-                        return 1;
+                        ENDFUNCRC(1);
                     }
-                    if (Count[0] == 0 && Count[1] == 0 && Count[2] == 0 && Count[3] == 0)
-                    {
+		    if (TEST_ZERO) {
                         free(StateMap);
-                        return 1;
+                        ENDFUNCRC(1);
                     }
                     Count[0]--;
                     break;
@@ -514,24 +602,29 @@ static int FindPrevIndent(EBuffer *B, int &RowP, int &ColP, char &CharP, int Fla
                         ColP = BolCol;
                         RowP = BolRow;
                         free(StateMap);
-                        return 1;
+                        ENDFUNCRC(1);
+                    }
+                    if (BolChar == ';') {
+                        CharP = BolChar;
+                        ColP = BolCol;
+                        RowP = BolRow;
+                        free(StateMap);
+                        ENDFUNCRC(1);
                     }
                     if (ColP == 0) { /* speed optimization */
                         free(StateMap);
-                        return 1;
+                        ENDFUNCRC(1);
                     }
-                    if (Count[0] == 0 && Count[1] == 0 && Count[2] == 0 && Count[3] == 0 &&
-                        (Flags & FIND_ENDBLOCK))
-                    {
+		    if (TEST_ZERO && (Flags & FIND_ENDBLOCK)) {
                         free(StateMap);
-                        return 1;
+                        ENDFUNCRC(1);
                     }
                     Count[0]++;
                     break;
                 case '(':
-                    if (Count[0] == 0 && Count[1] == 0 && Count[2] == 0 && Count[3] == 0) {
+		    if (TEST_ZERO) {
                         free(StateMap);
-                        return 1;
+                        ENDFUNCRC(1);
                     }
                     Count[1]--;
                     break;
@@ -539,16 +632,15 @@ static int FindPrevIndent(EBuffer *B, int &RowP, int &ColP, char &CharP, int Fla
                     Count[1]++;
                     break;
                 case '[':
-                    if (Count[0] == 0 && Count[1] == 0 && Count[2] == 0 && Count[3] == 0) {
+                    if (TEST_ZERO) {
                         free(StateMap);
-                        return 1;
+                        ENDFUNCRC(1);
                     }
                     Count[2]--;
                     break;
                 case ']':
                     Count[2]++;
                     break;
-
                 case ':':
                     if (ColP >= 1 && P[ColP - 1] == ':') { // skip ::
                         ColP -= 2;
@@ -556,21 +648,17 @@ static int FindPrevIndent(EBuffer *B, int &RowP, int &ColP, char &CharP, int Fla
                     }
                 case ',':
                 case ';':
-                    if (Count[0] == 0 &&
-                        Count[1] == 0 &&
-                        Count[2] == 0 &&
-                        Count[3] == 0 &&
-                        BolChar == ' ')
-                    {
-                        if ((CharP == ',' && (Flags & FIND_COMMA)) ||
-                            (CharP == ';' && (Flags & FIND_SEMICOLON)) ||
-                            (CharP == ':' && (Flags & FIND_COLON)))
-                        {
+                    if (TEST_ZERO && BolChar == ' ') {
+			if ((CharP == ';' && (Flags & FIND_SEMICOLON))
+			    || (CharP == ',' && (Flags & FIND_COMMA))
+			    || (CharP == ':' && (Flags & FIND_COLON))) {
                             BolChar = CharP;
                             BolCol = ColP;
-                            BolRow = RowP;
-                            //                            free(StateMap);
-                            //                            return 1;
+			    BolRow = RowP;
+			    // this should be here
+                            // if not say why ???
+			    //free(StateMap);
+			    //return 1;
                         }
                     }
                     if (BolChar == ',' && CharP == ':') {
@@ -579,12 +667,13 @@ static int FindPrevIndent(EBuffer *B, int &RowP, int &ColP, char &CharP, int Fla
                         BolRow = -1;
                         break;
                     }
-                    if ((BolChar == ':' || BolChar == ',') && (CharP == ';'/* || CharP == ','*/)) {
+		    if ((BolChar == ':' || BolChar == ',')
+			&& (CharP == ';'/* || CharP == ','*/)) {
                         CharP = ':';
                         ColP = BolCol;
                         RowP = BolRow;
                         free(StateMap);
-                        return 1;
+                        ENDFUNCRC(1);
                     }
                     break;
                 case '?':
@@ -606,10 +695,10 @@ static int FindPrevIndent(EBuffer *B, int &RowP, int &ColP, char &CharP, int Fla
                     if (Count[3] > 0)
                         Count[3]--;
                     if (Flags & FIND_IF) {
-                        if (Count[0] == 0 && Count[1] == 0 && Count[2] == 0 && Count[3] == 0) {
+                        if (TEST_ZERO) {
                             CharP = 'i';
                             free(StateMap);
-                            return 1;
+                            ENDFUNCRC(1);
                         }
                     }
                 }
@@ -619,15 +708,15 @@ static int FindPrevIndent(EBuffer *B, int &RowP, int &ColP, char &CharP, int Fla
                 {
                     //puts("\nelse\x7");
                     if (Flags & FIND_ELSE) {
-                        if (Count[0] == 0 && Count[1] == 0 && Count[2] == 0 && Count[3] == 0) {
+			if (TEST_ZERO) {
                             CharP = 'e';
                             free(StateMap);
-                            return 1;
+                            ENDFUNCRC(1);
                         }
                     }
                     Count[3]++;
                 }
-                if (Count[0] == 0 && Count[1] == 0 && Count[2] == 0 && Count[3] == 0) {
+		if (TEST_ZERO) {
 
                     if ((Flags & FIND_FOR) &&
                         L - ColP >= 3 &&
@@ -636,7 +725,7 @@ static int FindPrevIndent(EBuffer *B, int &RowP, int &ColP, char &CharP, int Fla
                     {
                         CharP = 'f';
                         free(StateMap);
-                        return 1;
+                        ENDFUNCRC(1);
                     }
                     if ((Flags & FIND_WHILE) &&
                         L - ColP >= 5 &&
@@ -645,7 +734,7 @@ static int FindPrevIndent(EBuffer *B, int &RowP, int &ColP, char &CharP, int Fla
                     {
                         CharP = 'w';
                         free(StateMap);
-                        return 1;
+                        ENDFUNCRC(1);
                     }
                     if ((Flags & FIND_SWITCH) &&
                         L - ColP >= 6 &&
@@ -654,7 +743,7 @@ static int FindPrevIndent(EBuffer *B, int &RowP, int &ColP, char &CharP, int Fla
                     {
                         CharP = 's';
                         free(StateMap);
-                        return 1;
+                        ENDFUNCRC(1);
                     }
                     if (((Flags & FIND_CASE) || (BolChar == ':')) &&
                         (L - ColP >= 4 &&
@@ -671,7 +760,7 @@ static int FindPrevIndent(EBuffer *B, int &RowP, int &ColP, char &CharP, int Fla
                             RowP = BolRow;
                         }
                         free(StateMap);
-                        return 1;
+                        ENDFUNCRC(1);
                     }
                     if (((Flags & FIND_CLASS) || (BolChar == ':')) &&
                         (L - ColP >= 5 &&
@@ -685,7 +774,7 @@ static int FindPrevIndent(EBuffer *B, int &RowP, int &ColP, char &CharP, int Fla
                             RowP = BolRow;
                         }
                         free(StateMap);
-                        return 1;
+                        ENDFUNCRC(1);
                     }
                     if (((Flags & FIND_CLASS) || (BolChar == ':')) &&
                         ((L - ColP >= 6 &&
@@ -705,7 +794,7 @@ static int FindPrevIndent(EBuffer *B, int &RowP, int &ColP, char &CharP, int Fla
                             RowP = BolRow;
                         }
                         free(StateMap);
-                        return 1;
+                        ENDFUNCRC(1);
                     }
                 }
             }
@@ -715,7 +804,7 @@ static int FindPrevIndent(EBuffer *B, int &RowP, int &ColP, char &CharP, int Fla
         if (BolChar != ' ' && BolChar != ':' && BolChar != ',') {
             CharP = BolChar;
             ColP = BolCol;
-            return 1;
+            ENDFUNCRC(1);
         }
         RowP--;
         if (RowP >= 0) {
@@ -723,7 +812,8 @@ static int FindPrevIndent(EBuffer *B, int &RowP, int &ColP, char &CharP, int Fla
             ColP = L - 1;
         }
     }
-    return 0;
+#undef TEST_ZERO
+    ENDFUNCRC(0);
 }
 
 #define SKIP_FORWARD  0
@@ -752,35 +842,34 @@ static int SkipWhite(EBuffer *B, int Bottom, int &Row, int &Col, int Flags) {
         if (B->GetMap(Row, &StateLen, &StateMap) == 0)
             return 0;
 
-        if (L > 0) for ( ;
-                         Col >= 0 && Col < L;
-                         Col += ((Flags & SKIP_BACK) ? -1 : +1))
-        {
-            if (P[Col] == ' ' || P[Col] == '\t')
-                continue;
-            if (StateMap[Col] != hsC_Normal &&
-                StateMap[Col] != hsC_Keyword &&
-                StateMap[Col] != hsC_String1 &&
-                StateMap[Col] != hsC_String2)
-                continue;
-            if (StateMap[Col] == hsC_Normal && (Flags & SKIP_MATCH)) {
-                switch (P[Col]) {
-                case '{': Count[0]--; continue;
-                case '}': Count[0]++; continue;
-                case '(': Count[1]--; continue;
-                case ')': Count[1]++; continue;
-                case '[': Count[2]--; continue;
-                case ']': Count[2]++; continue;
-                }
-            }
-            if (Count[0] == 0 && Count[1] == 0 && Count[2] == 0 && !(Flags & SKIP_TOBOL))
-            {
-                free(StateMap);
-                return 1;
-            }
-        }
-        free(StateMap);
-        if (Count[0] == 0 && Count[1] == 0 && Count[2] == 0 && (Flags & SKIP_TOBOL))
+	if (L > 0)
+	    for ( ; Col >= 0 && Col < L;
+		  Col += ((Flags & SKIP_BACK) ? -1 : +1)) {
+		if (P[Col] == ' ' || P[Col] == '\t')
+		    continue;
+		if (StateMap[Col] != hsC_Normal &&
+		    StateMap[Col] != hsC_Keyword &&
+		    StateMap[Col] != hsC_String1 &&
+		    StateMap[Col] != hsC_String2)
+		    continue;
+		if (StateMap[Col] == hsC_Normal && (Flags & SKIP_MATCH)) {
+		    switch (P[Col]) {
+		    case '{': Count[0]--; continue;
+		    case '}': Count[0]++; continue;
+		    case '(': Count[1]--; continue;
+		    case ')': Count[1]++; continue;
+		    case '[': Count[2]--; continue;
+		    case ']': Count[2]++; continue;
+		    }
+		}
+		if (Count[0] == 0 && Count[1] == 0 && Count[2] == 0
+		    && !(Flags & SKIP_TOBOL)) {
+		    free(StateMap);
+		    return 1;
+		}
+	    }
+	free(StateMap);
+	if (Count[0] == 0 && Count[1] == 0 && Count[2] == 0 && (Flags & SKIP_TOBOL))
             return 1;
         if (Flags & SKIP_LINE) {
             return 1;
@@ -803,6 +892,7 @@ static int SkipWhite(EBuffer *B, int Bottom, int &Row, int &Col, int Flags) {
 
 
 static int IndentNormal(EBuffer *B, int Line, int /*StateLen*/, hsState * /*StateMap*/) {
+    STARTFUNC("IndentNormal{h_c.cpp}");
     int I = 0;
     int Pos, L;
 
@@ -836,7 +926,8 @@ static int IndentNormal(EBuffer *B, int Line, int /*StateLen*/, hsState * /*Stat
         else
             return Pos;
     } else {
-        char CharP = ' ', FirstCharP = ' ';
+        char CharP = ' ';
+        // char FirstCharP = ' ';
         int RowP = Line;
         int ColP = -1;
         int PrevRowP = RowP;
@@ -849,7 +940,7 @@ static int IndentNormal(EBuffer *B, int Line, int /*StateLen*/, hsState * /*Stat
             return 0;
 
         PrevColP++;
-        PRINTF(("\nPrevRowP=%d, PrevColP=%d\n", PrevRowP, PrevColP));
+        LOG << "PrevRowP=" << PrevRowP << ", PrevColP=" << PrevColP << ENDLINE;
 
         if (FindPrevIndent(B, RowP, ColP, CharP,
                            FIND_IF |
@@ -876,9 +967,10 @@ static int IndentNormal(EBuffer *B, int Line, int /*StateLen*/, hsState * /*Stat
 
         FirstRowP = RowP;
         FirstColP = ColP;
-        FirstCharP = CharP;
+        // FirstCharP = CharP;
 
-        PRINTF(("\nFirstRowP=%d, FirstColP=%d, CharP=%c\n", FirstRowP, FirstColP, CharP));
+        LOG << "FirstRowP=" << FirstRowP << ", FirstColP=" << FirstColP <<
+            ", CharP=" << BinChar(CharP) << ENDLINE;
 
         switch (CharP) {
         case 'c':
@@ -893,7 +985,14 @@ static int IndentNormal(EBuffer *B, int Line, int /*StateLen*/, hsState * /*Stat
                 ColP++;
                 if (SkipWhite(B, Line, RowP, ColP, SKIP_FORWARD | SKIP_LINE) != 1)
                     return 0;
-                I = B->ScreenPos(B->RLine(RowP), ColP);
+                if (ColP < B->LineChars(RowP) || !FunctionUsesContinuation) {
+                    char strLeft[2] = { CharP, 0 };
+                    char strRight[2] = { CharP == '(' ? ')' : ']', 0 };
+                    I = SearchBackMatch(-1, B, Line - 1, hsC_Normal, strLeft, strRight, &Pos, &L);
+                    I = Pos + 1;
+                } else {
+                    I = B->LineIndented(RowP) + C_CONTINUATION;
+                }
             }
             return I;
 
@@ -911,7 +1010,7 @@ static int IndentNormal(EBuffer *B, int Line, int /*StateLen*/, hsState * /*Stat
                 I += C_FIRST_INDENT;
             else
                 I += C_INDENT;
-            PRINTF(("'{' indent : Line=%d, RowP=%d, ColP=%d, CharP=%c\n", Line, RowP, ColP, CharP));
+	    PRINTF(("'{' indent : Line=%d, RowP=%d, ColP=%d, CharP=%c\n", Line, RowP, ColP, CharP));
 
             if (LookAt(B, Line, 0, "{", hsC_Normal, 0))
                 I -= C_BRACE_OFS;
@@ -944,6 +1043,7 @@ static int IndentNormal(EBuffer *B, int Line, int /*StateLen*/, hsState * /*Stat
             }
             PRINTF(("';' Line=%d, RowP=%d, ColP=%d, CharP=%c\n", Line, RowP, ColP, CharP));
 
+            LOG << "  CharP now: " << BinChar(CharP) << ENDLINE;
             switch (CharP) {
             case ',':
             case ';':
@@ -997,8 +1097,12 @@ static int IndentNormal(EBuffer *B, int Line, int /*StateLen*/, hsState * /*Stat
             return I + ContinuationIndent;
 
         case ':':
-            ColP--;
-            if (FindPrevIndent(B, RowP, ColP, CharP, FIND_SEMICOLON | FIND_COLON | FIND_QUESTION | FIND_CLASS | FIND_CASE) != 1) {
+	    ColP--;
+            PRINTF(("COL-- %d\n", ColP));
+	    if (FindPrevIndent(B, RowP, ColP, CharP,
+			       FIND_SEMICOLON | FIND_COLON | FIND_QUESTION
+			       | FIND_CLASS | FIND_CASE) != 1) {
+		PRINTF(("FOUNPRE \n"));
                 if (FirstRowP != PrevRowP)
                     ContinuationIndent = C_CONTINUATION;
                 return 0 + ContinuationIndent;
@@ -1125,8 +1229,7 @@ static int IndentNormal(EBuffer *B, int Line, int /*StateLen*/, hsState * /*Stat
 
             return I + ContinuationIndent;
 
-        default:
-            return 0;
+        // default: return 0;
         }
     }
     return 0;
@@ -1195,8 +1298,10 @@ int Indent_C(EBuffer *B, int Line, int PosCursor) {
                     I = IndentCPP(B, Line, StateLen, 0);
                 } else {
                     I = IndentNormal(B, Line, StateLen, StateMap);
-                    if ((StateMap[0] == hsC_Comment || StateMap[0] == hsC_CommentL || StateMap[0] == hsC_CPP_Comm)
-                        && ((LookAt(B, Line, 0, "/*", hsC_Comment, 0)
+		    if ((StateMap[0] == hsC_Comment
+			 || StateMap[0] == hsC_CommentL
+			 || StateMap[0] == hsC_CPP_Comm)
+			&& ((LookAt(B, Line, 0, "/*", hsC_Comment, 0)
                              || LookAt(B, Line, 0, "/*", hsC_CPP_Comm, 0)
                              || LookAt(B, Line, 0, "//", hsC_CommentL, 0))))
                     {
