@@ -13,9 +13,8 @@
 
 int Hilit_SIMPLE(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Line, hlState &State, hsState *StateMap, int *ECol) {
     EColorize *col = BF->Mode->fColorize;
-    ChColor *Colors = col->Colors;
     HMachine *hm = col->hm;
-    HILIT_VARS(Colors[CLR_Normal], Line);
+    HILIT_VARS(col->Colors, Line);
     HState *st = 0;
     HTrans *tr = 0;
     int t, cc;
@@ -25,13 +24,13 @@ int Hilit_SIMPLE(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Li
     int nextState;
     char *match;
 
-    if (hm == 0 || col == 0 || hm->stateCount == 0)
+    if (hm == 0 || hm->stateCount == 0)
         return 0;
 
     if (State >= hm->stateCount)
         State = 0;
     st = hm->state + State;
-    Color = Colors[st->color];
+    Color = st->color;
 
     /*{
         fprintf(stderr, "ColMode:%s, State:%d\n", col->Name, State);
@@ -70,14 +69,22 @@ int Hilit_SIMPLE(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Li
                 nextState = tr->nextState;
 
                 //fprintf(stderr,
-                //        "line:%d, char:%d, len:%d, state:%d, tr:%d, st->transCount:%d, st->firstTrans, nextState:%d, matchFlags:%d\n",
-                //        LN, i, len, State, t, st->transCount, st->firstTrans, nextState, matchFlags);
+                //        "line:%d, char:%d (%c), len:%d, state:%d, tr:%d, st->transCount:%d, st->firstTrans:%d, nextState:%d, matchFlags:%08x\n",
+                //        LN, i, *p, len, State, t, st->transCount, st->firstTrans, nextState, matchFlags);
 
-               if (len < matchLen)
+                if (len < matchLen)
+                    continue;
+
+                if ((i > 0) && (matchFlags & MATCH_MUST_BOL))
                     continue;
 
                 if ((matchFlags & (MATCH_SET | MATCH_NOTSET)) == 0) {
-                    if (matchFlags & MATCH_NO_CASE) {
+                    if (matchFlags & MATCH_REGEXP) {
+                        RxMatchRes b;
+                        if (!RxExecMatch(tr->regexp, Line->Chars, Line->Count, p, &b, (matchFlags & MATCH_NO_CASE) ? 0 : RX_CASE)) continue;
+                        if (b.Open[1] != -1 && b.Close[1] != -1) matchLen = b.Open[1] - i;
+                        else matchLen = b.Close[0] - i;
+                    } else if (matchFlags & MATCH_NO_CASE) {
                         if (memicmp(match, p, matchLen))
                             continue;
                     } else {
@@ -93,9 +100,6 @@ int Hilit_SIMPLE(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Li
                         continue;
                 }
 
-                if ((i > 0) && (matchFlags & MATCH_MUST_BOL))
-                    continue;
-
                 if ((len != matchLen) && (matchFlags & MATCH_MUST_EOL))
                     continue;
 
@@ -112,7 +116,7 @@ int Hilit_SIMPLE(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Li
                             State = 0;
                         st = hm->state + State;
                     }
-                    Color = Colors[tr->color];
+                    Color = tr->color;
                     for (cc = 0; cc < matchLen; cc++)
                         IF_TAB()
                         else
@@ -144,7 +148,7 @@ int Hilit_SIMPLE(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Li
                        (WGETBIT(st->wordChars, Line->Chars[i + j]))) j++;
 
                 //GP (fix)
-                Color = Colors[st->color];
+                Color = st->color;
 
                 if (j == 0) {
                     if (st->nextKwdNoCharState != -1) {
@@ -152,12 +156,12 @@ int Hilit_SIMPLE(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Li
                         if (State >= hm->stateCount)
                             State = 0;
                         st = hm->state + State;
-                        Color = Colors[st->color];
+                        Color = st->color;
                         goto next_state;
                     }
                 } else {
-                    if (st->GetHilitWord(j, &Line->Chars[i], Color) ||
-                        BF->GetHilitWord(j, &Line->Chars[i], Color, (st->options & STATE_NOCASE) ? 1 : 0))
+                    if (st->GetHilitWord(j, &Line->Chars[i], Color ) ||
+                        BF->GetHilitWord(j, &Line->Chars[i], Color,  BFI( BF, BFI_MatchCase ) ? 0 : 1))
                     {
                         if (st->nextKwdMatchedState != -1)
                             State = st->nextKwdMatchedState;
@@ -168,7 +172,7 @@ int Hilit_SIMPLE(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Li
                                 if (State >= hm->stateCount)
                                     State = 0;
                                 st = hm->state + State;
-                                Color = Colors[st->color];
+                                Color = st->color;
                                 goto next_state;
                             }
                         }
@@ -181,13 +185,13 @@ int Hilit_SIMPLE(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Li
                     if (st->options & STATE_TAGASNEXT) {
                         MState = State;
                         st = hm->state + State;
-                        Color = Colors[st->color];
+                        Color = st->color;
                     }
                     
                     if (StateMap)
                         memset(StateMap + i, MState, j);
                     if (B)
-                        MoveMem(B, C - Pos, Width, Line->Chars + i, Color, j);
+                        MoveMem(B, C - Pos, Width, Line->Chars + i, HILIT_CLRD(), j);
                     i += j;
                     len -= j;
                     p += j;
@@ -195,13 +199,13 @@ int Hilit_SIMPLE(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Li
 
                     if (!(st->options & STATE_TAGASNEXT)) {
                         st = hm->state + State;
-                        Color = Colors[st->color];
+                        Color = st->color;
                     }
                     goto next_state;
                 }
             }
         }
-        Color = Colors[st->color];
+        Color = st->color;
         IF_TAB()
         else
             ColorNext();
@@ -217,7 +221,7 @@ int Hilit_SIMPLE(EBuffer *BF, int /*LN*/, PCell B, int Pos, int Width, ELine *Li
         match = tr->match;
         nextState = tr->nextState;
         
-        if ((i > 0) && (matchFlags & MATCH_MUST_BOL))
+        if (((i > 0) && (matchFlags & MATCH_MUST_BOL)) || (matchFlags & MATCH_REGEXP))
             continue;
 
         //cant match eol beyond eol.
